//Copyright 2013 Katarzyna Kułakowska, Tomasz Kułakowski.
//
//This file is part of SkyPig.
//
//SkyPig is free software; you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation; either version 2 of the License, or
//(at your option) any later version.
//
//SkyPig is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with SkyPig; if not, write to the Free Software
//Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

package model.core;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.*;

/**
 *  Stores all parameters read from a specified file and provides them to all.
 *
 *  @author Tomasz Kułakowski
 *
 */
public class Config {
	/*
	 *  Fields:
	 */
	/**
	 * Stores a random number generator available to all the classes. <p> May be
	 * initialized using the seed specified in the configuration file. </p>
	 *
	 * @since 2012-08-06
	 */
	private final Random mainGenerator;

	/**
	 *
	 */
	private static Config configuration = null;
	/**
	 *
	 */
	private HashMap<String, Object> parameterList;

	/*
	 *  Methods:
	 */
	/**
	 *  Checks if
	 *  <code>configuration</code> has already been initialised and initialises
	 * it if not.
	 *
	 *  @param configFile
	 *  @return true if initialisation has been performed and false if it has not
	 * been necessary.
	 */
	public static boolean initialise(String configFile) {
		if (configuration == null) {
			configuration = new Config(configFile);
			Calendar time = new GregorianCalendar();
			configuration.parameterList.put("startTime", String.valueOf(time.getTimeInMillis()) + "_" + String.format("%05d",configuration.mainGenerator.nextInt(10000)));
			configuration.parameterList.put("configurationFile", configFile);
			if (configuration.parameterList.get("World_randomSeed") == null) {
				configuration.parameterList.put("World_randomSeed", new Random().nextLong());
			}
			return true;
		} else {
			return false;
		}
	}

	public static Random getGenerator() {
		return configuration.mainGenerator;
	}
	/**
	 *  Returns a parameter of given name or null if it could not be accessed; in
	 * the latter case prints an appropriate error message.
	 *
	 *  @param parameter
	 *  @return An object containing a value of the parameter of a given name or
	 * null if it could not be accessed.
	 */
	public static Object getParameter(String parameter) {
		if (configuration != null) {
			if (configuration.parameterList.containsKey(parameter)) {
				return configuration.parameterList.get(parameter);
			} else {
				System.out.println("Error: Parameter " + parameter + " not found on parameterList");
				return null;
			}
		} else {
			System.out.println("Error: Configuration not initialised.");
			return null;
		}
	}
	
	public static HashMap<String, Object> getParameterList() {
		HashMap<String, Object> copy =  new HashMap<String, Object>();
		copy.putAll(configuration.parameterList);
		return copy;
	}
	
	public static boolean checkIfPesticideApplicationYear(String pesticide, String name) {
			if ((World.getDate().get(GregorianCalendar.YEAR) - ((GregorianCalendar) Config.getParameter("World_startDate")).get(GregorianCalendar.YEAR) >= (Integer) Config.getParameter("Config_burnin"))) {
				if (((Integer) ((HashMap<String, HashMap<String, Object>>) Config.getParameter(name + "_pesticide")).get(pesticide).get("year")) == null) {
					return true;
				} else if (World.getDate().get(GregorianCalendar.YEAR)
						== (((Integer) ((HashMap<String, HashMap<String, Object>>) Config.getParameter(name + "_pesticide")).get(pesticide).get("year")))) {
					return true;
				}
				return false;
			} else {
				return false;
			}
		}

	/**
	 *
	 *  @param date
	 *  @return
	 */
	public static int getDayLength(Calendar date) {
		Double dayLength = (Double) ((ArrayList<Double>) getParameter("World_dayLength")).get(date.get(Calendar.DAY_OF_YEAR) - 1);
		double dayMinutes = ((Double) Math.floor(dayLength)) * 60 + (dayLength - (Double) Math.floor(dayLength)) * 100;
		return (int) Math.round(dayMinutes / ((Integer) getParameter("Config_timeFactor")).doubleValue());
	}

	/**
	 *
	 *  @param date
	 *  @return
	 */
	public static double getRealDayLength(Calendar date) {
		return (Double) ((ArrayList<Double>) getParameter("World_dayLength")).get(date.get(Calendar.DAY_OF_YEAR) - 1);
	}

	/**
	 *
	 *  @return
	 */
	public static double getLongestDayRealLength() {
		return Collections.max((ArrayList<Double>) getParameter("World_dayLength"));
	}

	/**
	 *
	 *  @return
	 */
	public static int getLongestDayLength() {
		double dayLength = Collections.max((ArrayList<Double>) getParameter("World_dayLength"));
		double dayMinutes = ((Double) Math.floor(dayLength)) * 60 + (dayLength - (Double) Math.floor(dayLength)) * 100;
		return (int) Math.round(dayMinutes / ((Integer) getParameter("Config_timeFactor")).doubleValue());
	}

	/**
	 *
	 *  @return
	 */
	public static String getLogInfo() {
		String a = "\n configuration number: " + (String) getParameter("configurationName");
		a += "\n Run start time: " + getParameter("startTime");
		a += "\n date: " + Integer.toString(World.getDate().get(Calendar.YEAR)) + "-"
				+ Integer.toString(World.getDate().get(Calendar.MONTH) + 1) + "-"
				+ Integer.toString(World.getDate().get(Calendar.DAY_OF_MONTH));
		return a;
	}

	/**
	 *
	 *  @return
	 */
	public static int getDensityDependentEmigration() {
		int adults = World.Instance().getNumberOfAdultsAndNests()[0];
		return (int) Math.ceil(adults * (Double) getParameter("Config_densityDependentMigration"));
	}

	/**
	 *  Constructor called only in
	 *  <code>initialise</code>; reads parameters from the specified file.
	 *
	 *  @param fileName
	 */
	private Config(String fileName) {
		parameterList = new HashMap<String, Object>();
		readParameters(fileName);
		mainGenerator = new Random();//(Long) getParameter("World_randomSeed"));
	}

	/**
	 *  Reads parameters from file of given name (searches in current directory
	 * only) to given HashMaps.
	 *
	 *  @param filename
	 */
	private void readParameters(String filename/*
			 *  , HashMap<String, Object> parameterList, HashMap<Number,
			 * HashMap<String, Number>> changesList
			 */) {
		BufferedReader in = null;
		try {
			in = new BufferedReader(new FileReader(filename));
		} catch (FileNotFoundException e) {
			e.printStackTrace();
			System.out.println("Configuration file not found. Aborting reading.");
			return;
		}
		String line;
		Scanner myScanner;
		double realValue = 0.0;
		int intValue = 0;
		long longValue = 0;
		String name;
		String type;
		Integer lineNumber = 0;
		try {
			while ((line = in.readLine()) != null) {
				myScanner = new Scanner(line);
				lineNumber++;
				if (myScanner.hasNext()) {
					name = myScanner.next();
					if (name.startsWith("#")) {
						continue;
					}
				} else {
					continue;
				}
				if (myScanner.hasNext()) {
					type = myScanner.next();
				} else {
					continue;
				}
				if (type.equalsIgnoreCase("double")) {
					if (myScanner.hasNextDouble()) {
						realValue = myScanner.nextDouble();
						parameterList.put(name, realValue);
					} else {
						System.out.println("Error: missing double value.");
						System.out.println("Variable: " + name + " line number: " + lineNumber.toString() + ".");
					}
				} else if (type.equalsIgnoreCase("integer")) {
					if (myScanner.hasNextInt()) {
						intValue = myScanner.nextInt();
						parameterList.put(name, intValue);
					} else if (myScanner.hasNext()) {
						String value = myScanner.next();
						if (value.equalsIgnoreCase("null")) {
							parameterList.put(name, null);
						}
					} else {
						System.out.println("Error: missing integer value.");
						System.out.println("Variable: " + name + " line number: " + lineNumber.toString() + ".");
					}
				} else if (type.equalsIgnoreCase("long")) {
					if (myScanner.hasNextLong()) {
						longValue = myScanner.nextLong();
						parameterList.put(name, longValue);
					} else if (myScanner.hasNext()) {
						String value = myScanner.next();
						if (value.equalsIgnoreCase("null")) {
							parameterList.put(name, null);
						}
					} else {
						System.out.println("Error: missing long value.");
						System.out.println("Variable: " + name + " line number: " + lineNumber.toString() + ".");
					}
				} else if (type.equalsIgnoreCase("string")) {
					if (myScanner.hasNext()) {
						parameterList.put(name, myScanner.next());
					} else {
						System.out.println("Error: missing string value.");
						System.out.println("Variable: " + name + " line number: " + lineNumber.toString() + ".");
					}
				} else if (type.equalsIgnoreCase("list_of_doubles")) {
					if (myScanner.hasNext()) {
						String listElements = myScanner.next();
						ArrayList<Double> doubleList = new ArrayList<Double>();
						Scanner listScanner = new Scanner(listElements).useDelimiter(",");
						while (listScanner.hasNextDouble()) {
							doubleList.add(listScanner.nextDouble());
						}
						parameterList.put(name, doubleList);
					}
				} else if (type.equalsIgnoreCase("list_of_integers")) {
					if (myScanner.hasNext()) {
						String listElements = myScanner.next();
						ArrayList<Integer> integerList = new ArrayList<Integer>();
						Scanner listScanner = new Scanner(listElements).useDelimiter(",");
						while (listScanner.hasNextInt()) {
							integerList.add(listScanner.nextInt());
						}
						parameterList.put(name, integerList);
					}
				} else if (type.equalsIgnoreCase("list_of_strings")) {
					if (myScanner.hasNext()) {
						String listElements = myScanner.next();
						ArrayList<String> stringList = new ArrayList<String>();
						Scanner listScanner = new Scanner(listElements).useDelimiter(",");
						while (listScanner.hasNext()) {
							String element = listScanner.next();
							if (element.equalsIgnoreCase("null")) {
								element = null;
							}
							stringList.add(element);
						}
						parameterList.put(name, stringList);
					}
				} else if (type.equalsIgnoreCase("list_of_dates")) {
					if (myScanner.hasNext()) {
						String listElements = myScanner.next();
						ArrayList<GregorianCalendar> dateList = new ArrayList<GregorianCalendar>();
						Scanner listScanner = new Scanner(listElements).useDelimiter(",");
						while (listScanner.hasNext()) {
							Scanner dateScanner = new Scanner(listScanner.next()).useDelimiter("-");
							ArrayList<Integer> integerList = new ArrayList<Integer>();
							if (dateScanner.hasNextInt()) {
								while (dateScanner.hasNextInt()) {
									integerList.add(dateScanner.nextInt());
								}
								if (integerList.size() == 3) {
									dateList.add(new GregorianCalendar(integerList.get(0), integerList.get(1) - 1, integerList.get(2)));
								} else if (integerList.size() == 2) {
									dateList.add(new GregorianCalendar(0, integerList.get(1) - 1, integerList.get(2)));
								}
							} else if (dateScanner.hasNext()) {
								String value = dateScanner.next();
								if (value.equalsIgnoreCase("null")) {
									dateList = null;
								}
							}
						}
						parameterList.put(name, dateList);
					}
				} else if (type.equalsIgnoreCase("boolean")) {
					if (myScanner.hasNext()) {
						String value = myScanner.next();
						if (value.compareToIgnoreCase("true") == 0) {
							parameterList.put(name, true);
						} else if (value.compareToIgnoreCase("false") == 0) {
							parameterList.put(name, false);
						} else {
							System.out.println("Error: Illegal value for boolean variable.");
							System.out.println("Variable: " + name + " line number: " + lineNumber.toString() + ".");
						}
					}
				} else if (type.equalsIgnoreCase("date")) {
					if (myScanner.hasNext()) {
						String listElements = myScanner.next();
						ArrayList<Integer> integerList = new ArrayList<Integer>();
						Scanner listScanner = new Scanner(listElements).useDelimiter("-");
						while (listScanner.hasNextInt()) {
							integerList.add(listScanner.nextInt());
						}
						parameterList.put(name, new GregorianCalendar(integerList.get(0), integerList.get(1) - 1, integerList.get(2)));
					}
				} else if (type.equalsIgnoreCase("distribution_of_integers")) {
					TreeMap<Double, Integer> cumulativeDistribution = new TreeMap<Double, Integer>();
					double cumulativeProbability = 0.0;
					Scanner newScanner = new Scanner(myScanner.next()).useDelimiter(",");
					while (newScanner.hasNextInt()) {
						intValue = newScanner.nextInt();
						if (newScanner.hasNextDouble()) {
							cumulativeProbability += newScanner.nextDouble();
							cumulativeDistribution.put(cumulativeProbability, intValue);
						} else {
							System.out.println("Error: Missing probability value in integer distribution.");
							System.out.println("Variable: " + name + " line number: " + lineNumber.toString() + ".");
							break;
						}
					}
					parameterList.put(name, cumulativeDistribution);
				} else if (type.equalsIgnoreCase("distribution_of_doubles")) {
					TreeMap<Double, Double> cumulativeDistribution = new TreeMap<Double, Double>();
					double cumulativeProbability = 0.0;
					Scanner newScanner = new Scanner(myScanner.next()).useDelimiter(",");
					while (newScanner.hasNextDouble()) {
						realValue = newScanner.nextDouble();
						if (newScanner.hasNextDouble()) {
							cumulativeProbability += newScanner.nextDouble();
							cumulativeDistribution.put(cumulativeProbability, realValue);
						} else {
							System.out.println("Error: Missing probability value in integer distribution.");
							System.out.println("Variable: " + name + " line number: " + lineNumber.toString() + ".");
							break;
						}
					}
					parameterList.put(name, cumulativeDistribution);
				} else if (type.equalsIgnoreCase("description")) {
					if (myScanner.hasNext()) {
						String listElements = myScanner.next();
						HashMap<String, HashMap<String, Object>> List = new HashMap<String, HashMap<String, Object>>();
						if (parameterList.containsKey(name)) {
							List = (HashMap<String, HashMap<String, Object>>) parameterList.get(name);
						}
						Scanner listScanner = new Scanner(listElements).useDelimiter(",");
						String listName = listScanner.next();
						HashMap<String, Object> listContent = new HashMap<String, Object>();
						if (listScanner.hasNext()) {
							listContent = lineToMap(listScanner.next(), listScanner);
						}
						List.put(listName, listContent);
						if (listName.equalsIgnoreCase("null")) {
							parameterList.put(name, null);
						} else {
							parameterList.put(name, List);
						}
					}
				} else {
					System.out.println("Error: Unrecognised parameter type.");
					System.out.println(type);
					System.out.println("Variable: " + name + " line number: " + lineNumber.toString() + ".");
				}
			}
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			System.out.println("IOException during reading of configuration file. Aborting.");
		}
	}

	/**
	 *
	 *  @param firstWord
	 *  @param myScanner
	 *  @return
	 */
	public static HashMap<String, Object> lineToMap(String firstWord,
			Scanner myScanner) {
		HashMap<String, Object> map = new HashMap<String, Object>();
		while (myScanner.hasNext()) {
			String type = myScanner.next();
			if (type.compareToIgnoreCase("double") == 0) {
				if (myScanner.hasNextDouble()) {
					map.put(firstWord, myScanner.nextDouble());
				}
			} else if (type.compareToIgnoreCase("integer") == 0) {
				if (myScanner.hasNextInt()) {
					map.put(firstWord, myScanner.nextInt());
				}
			} else if (type.compareToIgnoreCase("string") == 0) {
				if (myScanner.hasNext()) {
					map.put(firstWord, myScanner.next());
				}
			} else if (type.compareToIgnoreCase("list_of_doubles") == 0) {
				if (myScanner.hasNext()) {
					String listElements = myScanner.next();
					ArrayList<Double> doubleList = new ArrayList<Double>();
					Scanner listScanner = new Scanner(listElements).useDelimiter(",");
					while (listScanner.hasNextDouble()) {
						doubleList.add(listScanner.nextDouble());
					}
					map.put(firstWord, doubleList);
				}
			} else if (type.compareToIgnoreCase("list_of_integers") == 0) {
				if (myScanner.hasNext()) {
					String listElements = myScanner.next();
					ArrayList<Integer> integerList = new ArrayList<Integer>();
					Scanner listScanner = new Scanner(listElements).useDelimiter(",");
					while (listScanner.hasNextInt()) {
						integerList.add(listScanner.nextInt());
					}
					map.put(firstWord, integerList);
				}
			} else if (type.compareToIgnoreCase("list_of_strings") == 0) {
				if (myScanner.hasNext()) {
					String listElements = myScanner.next();
					ArrayList<String> stringList = new ArrayList<String>();
					Scanner listScanner = new Scanner(listElements).useDelimiter(",");
					while (listScanner.hasNext()) {
						stringList.add(listScanner.next());
					}
					map.put(firstWord, stringList);
				}
			} else if (type.compareToIgnoreCase("boolean") == 0) {
				if (myScanner.hasNext()) {
					String value = myScanner.next();
					if (value.compareToIgnoreCase("true") == 0) {
						map.put(firstWord, true);
					} else if (value.compareToIgnoreCase("false") == 0) {
						map.put(firstWord, false);
					} else {
						System.out.println("Error: Illegal value for boolean variable.");
					}
				}
			} else {
				System.out.print(type);
				System.out.println("Unrecognised parameter type.");
			}
			if (myScanner.hasNext()) {
				firstWord = myScanner.next();
			}
		}
		return map;
	}
}
