//Copyright 2013 Katarzyna Kułakowska, Tomasz Kułakowski.
//
//This file is part of SkyPig.
//
//SkyPig is free software; you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation; either version 2 of the License, or
//(at your option) any later version.
//
//SkyPig is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with SkyPig; if not, write to the Free Software
//Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
/**
 *
 */
package model.core;

import java.lang.reflect.Type;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.Collections;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * @author Tomasz Kułakowski
 * @author Katarzyna Matuszewska
 *
 */
public class ExposureObserver extends Observer {

	private HashMap<String, LinkedList<Double>> nestlingDose;
	private HashMap<String, LinkedList<Double>> adultDose;
	private HashMap<String, LinkedList<Integer>> nestlingTime;
	private HashMap<String, LinkedList<Integer>> adultTime;
	private HashMap<String, LinkedList<Double>> nestlingAmount;
	private HashMap<String, LinkedList<Double>> adultAmount;
	private ArrayList<String> pesticides;

	/**
	 * @param databaseName
	 */
	public ExposureObserver(String databaseName) {
		super(databaseName);
		nestlingDose = new HashMap<String, LinkedList<Double>>();
		adultDose = new HashMap<String, LinkedList<Double>>();
		nestlingTime = new HashMap<String, LinkedList<Integer>>();
		adultTime = new HashMap<String, LinkedList<Integer>>();
		nestlingAmount = new HashMap<String, LinkedList<Double>>();
		adultAmount = new HashMap<String, LinkedList<Double>>();
		pesticides = (ArrayList<String>) Config.getParameter("Pesticides");
		resetPesticideIndices();
	}

	public void observe(Collection<Observable> observed) {
		if (World.getDate().get(Calendar.DAY_OF_YEAR)
				== (Integer) Config.getParameter("DisturbanceObserver_annualSaveDay") //skylarks: SkylarkAnimal_emigration day - 2
				&& World.getDate().get(Calendar.YEAR)
				== (Integer) Config.getParameter("WoodpigeonLandscapeUnit_pesticideYear") + 1) {
			try {
				for (Observable o : observed) {
					try {
						Animal a = (Animal) o;
						for (String pesticide : a.getMaxPesticides().keySet()) {
							//pesticides.add(pesticide);
							if (!nestlingDose.containsKey(pesticide)) {
								nestlingDose.put(pesticide, new LinkedList<Double>());
							}
							if (a.getDateOfLaying().get(GregorianCalendar.YEAR) == World.getDate().get(GregorianCalendar.YEAR)) {
								nestlingDose.get(pesticide).add(a.getMaxPesticides().get(pesticide));
							} else {
								if (!adultDose.containsKey(pesticide)) {
									adultDose.put(pesticide, new LinkedList<Double>());
								}
								adultDose.get(pesticide).add(a.getMaxPesticides().get(pesticide));
							}
						}
						for (String pesticide : a.getExposureTime().keySet()) {
							//pesticides.add(pesticide);
							if (a.getDateOfLaying().get(GregorianCalendar.YEAR) == World.getDate().get(GregorianCalendar.YEAR)) {
								if (!nestlingTime.containsKey(pesticide)) {
									nestlingTime.put(pesticide, new LinkedList<Integer>());
								}
								nestlingTime.get(pesticide).add(a.getExposureTime().get(pesticide));
							} else {
								if (!adultTime.containsKey(pesticide)) {
									adultTime.put(pesticide, new LinkedList<Integer>());
								}
								adultTime.get(pesticide).add(a.getExposureTime().get(pesticide));
							}
						}
						for (String pesticide : a.getCumulativePesticides().keySet()) {
							//pesticides.add(pesticide);
							if (a.getDateOfLaying().get(GregorianCalendar.YEAR) == World.getDate().get(GregorianCalendar.YEAR)) {
								if (!nestlingAmount.containsKey(pesticide)) {
									nestlingAmount.put(pesticide, new LinkedList<Double>());
								}
								nestlingAmount.get(pesticide).add(a.getCumulativePesticides().get(pesticide));
							} else {
								if (!adultAmount.containsKey(pesticide)) {
									adultAmount.put(pesticide, new LinkedList<Double>());
								}
								adultAmount.get(pesticide).add(a.getCumulativePesticides().get(pesticide));
							}
						}
					} catch (Exception e1) {
						try {
							Nest n = (Nest) o;
							for (HashMap<String, Double> maxP : n.getOffspringMaxPesticides().values()) {
								for (String pesticide : maxP.keySet()) {
									//pesticides.add(pesticide);
									if (!nestlingDose.containsKey(pesticide)) {
										nestlingDose.put(pesticide, new LinkedList<Double>());
									}
									nestlingDose.get(pesticide).add(maxP.get(pesticide));
								}
							}
							for (HashMap<String, Integer> expT : n.getOffspringExposureTime().values()) {
								for (String pesticide : expT.keySet()) {
									//pesticides.add(pesticide);
									if (!nestlingTime.containsKey(pesticide)) {
										nestlingTime.put(pesticide, new LinkedList<Integer>());
									}
									nestlingTime.get(pesticide).add(expT.get(pesticide));
								}
							}
							for (HashMap<String, Double> cumP : n.getOffspringCumulativePesticides().values()) {
								for (String pesticide : cumP.keySet()) {
									//pesticides.add(pesticide);
									if (!nestlingAmount.containsKey(pesticide)) {
										nestlingAmount.put(pesticide, new LinkedList<Double>());
									}
									nestlingAmount.get(pesticide).add(cumP.get(pesticide));
								}
							}
						} catch (Exception e2) {
							//DO NOTHING
						}
					}
				}
				for (String pesticide : pesticides) {
					makeTable(pesticide);
					int year = World.getDate().get(GregorianCalendar.YEAR);
					String update = "INSERT INTO exposure_" + pesticide + " (starttime, year, nestling_dose, adult_dose, nestling_time, adult_time, nestling_amount, adult_amount) VALUES ('";
					update += Config.getParameter("startTime") + "', ";
					update += String.valueOf(year) + ", ";
					if (nestlingDose.containsKey(pesticide)) {
						update += "'{";
						Collections.sort(nestlingDose.get(pesticide));
						for (Double dose : nestlingDose.get(pesticide)) {
							update += String.valueOf(dose) + ", ";
						}
						if (update.endsWith(", ")) {
							update = update.substring(0, update.length() - 2) + "}', ";
						} else {
							update = update + "}', ";
						}
					} else {
						update += "null, ";
					}
					if (adultDose.containsKey(pesticide)) {
						update += "'{";
						Collections.sort(adultDose.get(pesticide));
						for (Double dose : adultDose.get(pesticide)) {
							update += String.valueOf(dose) + ", ";
						}
						if (update.endsWith(", ")) {
							update = update.substring(0, update.length() - 2) + "}', ";
						} else {
							update = update + "}', ";
						}
					} else {
						update += "null, ";
					}
					if (nestlingTime.containsKey(pesticide)) {
						update += "'{";
						Collections.sort(nestlingTime.get(pesticide));
						for (Integer time : nestlingTime.get(pesticide)) {
							update += String.valueOf(time) + ", ";
						}
						if (update.endsWith(", ")) {
							update = update.substring(0, update.length() - 2) + "}', ";
						} else {
							update = update + "}', ";
						}
					} else {
						update += "null, ";
					}
					if (adultTime.containsKey(pesticide)) {
						update += "'{";
						Collections.sort(adultTime.get(pesticide));
						for (Integer time : adultTime.get(pesticide)) {
							update += String.valueOf(time) + ", ";
						}
						if (update.endsWith(", ")) {
							update = update.substring(0, update.length() - 2) + "}', ";
						} else {
							update = update + "}', ";
						}
					} else {
						update += "null, ";
					}
					if (nestlingAmount.containsKey(pesticide)) {
						update += "'{";
						Collections.sort(nestlingAmount.get(pesticide));
						for (Double dose : nestlingAmount.get(pesticide)) {
							update += String.valueOf(dose) + ", ";
						}
						if (update.endsWith(", ")) {
							update = update.substring(0, update.length() - 2) + "}', ";
						} else {
							update = update + "}', ";
						}
					} else {
						update += "null, ";
					}
					if (adultAmount.containsKey(pesticide)) {
						update += "'{";
						Collections.sort(adultAmount.get(pesticide));
						for (Double dose : adultAmount.get(pesticide)) {
							update += String.valueOf(dose) + ", ";
						}
						if (update.endsWith(", ")) {
							update = update.substring(0, update.length() - 2) + "}');";
						} else {
							update = update + "}');";
						}
					} else {
						update += "null);";
					}
					addBatch(update);
				}
				executeBatch();
				resetPesticideIndices();
			} catch (SQLException e) {
				Logger.getLogger(Observer.class.getName()).log(Level.SEVERE, null,
						e);
				Logger.getLogger(Observer.class.getName()).log(Level.SEVERE, null,
						e.getNextException());
			}
		}
	}

	private void makeTable(String pesticide) {
		try {
			if (!getTables("exposure_" + pesticide).next()) {
				executeStatement("CREATE TABLE IF NOT EXISTS exposure_" + pesticide + " (starttime text, year integer, nestling_dose double precision[], adult_dose double precision[], nestling_time integer[], adult_time integer[], nestling_amount double precision[], adult_amount double precision[], PRIMARY KEY (starttime, year));");
			}
		} catch (SQLException e) {
			e.printStackTrace();
			e.getNextException().printStackTrace();
		}
	}

	private void resetPesticideIndices() {
		for (String pesticide : (ArrayList<String>) Config.getParameter("Pesticides")) {
			nestlingDose.put(pesticide, new LinkedList<Double>());
			adultDose.put(pesticide, new LinkedList<Double>());
			nestlingTime.put(pesticide, new LinkedList<Integer>());
			adultTime.put(pesticide, new LinkedList<Integer>());
			nestlingAmount.put(pesticide, new LinkedList<Double>());
			adultAmount.put(pesticide, new LinkedList<Double>());
		}
	}
}
