//Copyright 2013 Katarzyna Kułakowska, Tomasz Kułakowski.
//
//This file is part of SkyPig.
//
//SkyPig is free software; you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation; either version 2 of the License, or
//(at your option) any later version.
//
//SkyPig is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with SkyPig; if not, write to the Free Software
//Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

package model.core;

import java.util.HashSet;
import java.util.Set;
import java.util.concurrent.ConcurrentLinkedQueue;
import java.util.logging.Level;

/**
 * Serves for storing the incoming Messages.
 *
 * @author Katarzyna Kułakowska
 * @author Tomasz Kułakowski
 * @version 1.0
 */
public class Inbox {

    /**
     * Collection for storing Messages.
     */
    private ConcurrentLinkedQueue<Message> storage;

    /**
     * Creates a newInbox with an empty storage.
     */
    public Inbox() {
        storage = new ConcurrentLinkedQueue<Message>();
    }

    /**
     * Enables adding messages to this inbox.
     *
     * @param message   to be added to this Inbox storage
     * @verification TEST 1 Warning if the message is added to the storage twice.
     */
    public void addMessage(Message message) {
//*****VERIFICATION MODE START*****//
        if (World.verificationMode) {
            //***TEST 1
            if (storage.contains(message)) {
                try {
                    throw new SkyPigException("Message added to inbox twice: "
                            + message.topic);
                } catch (SkyPigException ex) {
                    World.log(Level.WARNING, Config.getLogInfo(), ex);
                }
            }
        }
//*****VERIFICATION MODE END*****//
        storage.add(message);
        return;
    }

    /**
     * Checks if the inbox contains any messages.
     *
     * @return true if the storage is empty, false otherwise
     */
    public boolean isEmpty() {
        return storage.isEmpty();
    }

    /**
     * Returns one Message from the storage.
     *
     * @return the first Message in the queue, or null if the storage is empty
     */
    public Message getMessage() {
        return storage.poll();
    }

    /**
     * Finds in the storage and returns Messages with a specified topic,
     * leaving their copies in the inbox.
     *
	 * @param sender 
     * @return set of Messages with the specified topic or an empty Set if
     * Messages with this topic were not found
     */
    public Set<Message> findMessage(Long sender) {
        HashSet<Message> set = new HashSet();
        for (Message m : storage) {
            if (m.senderId.equals(sender)) {
                set.add(m);
            }
        }
        return set;
    }
}
