//Copyright 2013 Katarzyna Kułakowska, Tomasz Kułakowski.
//
//This file is part of SkyPig.
//
//SkyPig is free software; you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation; either version 2 of the License, or
//(at your option) any later version.
//
//SkyPig is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with SkyPig; if not, write to the Free Software
//Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
package model.core;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.logging.Level;

import model.skylark.SkylarkCalibrationObserver;
import model.woodpigeon.DisturbanceObserver;
import model.woodpigeon.SensitivityObserver;

/**
 * The main class of the model.
 *
 * @author Tomasz Kułakowski
 * @author Katarzyna Kułakowska
 * @since 2013-01-03
 *
 */
public class Model {

	/**
	 * Runs the model.
	 *
	 * @param args - filename of configuration file containing values of
	 * parameters - args[0].
	 * @since 2013-01-04
	 */
	public static void main(String[] args) {
		if (args.length == 1) {
			Config.initialise(args[0]);
		} else {
			System.out.print("Wrong parameters given. Quitting.");
			return;
		}
		Observer observer = null;
		HashSet<Observer> observers = new HashSet<Observer>();
		for (String type : (ArrayList<String>) Config.getParameter("Model_observers")) {
			//String type = (String) Config.getParameter("Model_type");
			if (type != null) {
				if (type.equalsIgnoreCase("specific")) {
					observer = new SpecificObserver(
							(String) Config.getParameter("World_dataBaseName"));
				} else if (type.equalsIgnoreCase("exposure")) {
					observer = new ExposureObserver(
							(String) Config.getParameter("World_dataBaseName"));
				} else if (type.equalsIgnoreCase("effects")) {
					observer = new EffectObserver(
							(String) Config.getParameter("World_dataBaseName"));
				} else if (type.equalsIgnoreCase("disturbance")) {
					observer = new DisturbanceObserver(
							(String) Config.getParameter("World_dataBaseName"));
				} else if (type.equalsIgnoreCase("debug")) {
					observer = new Observer(
							(String) Config.getParameter("World_dataBaseName"));
				} else if (type.equalsIgnoreCase("sensitivity")) {
					observer = new SensitivityObserver(
							(String) Config.getParameter("World_dataBaseName"));
				} else if (((String) Config.getParameter("World_speciesName")).equalsIgnoreCase("skylark")
						&& type.equalsIgnoreCase("calibration")) {
					observer = new SkylarkCalibrationObserver(
							(String) Config.getParameter("World_dataBaseName"));
				} else {
					System.out.println("Unrecognised model observer: " + type);
				}
				if (observer != null) {
					observers.add(observer);
				}
			}
		}
		if (observers.isEmpty()) {
			System.out.println("Model execution without observer.");
		}
		for (Observer obs : observers) {
			obs.storeConfiguration(args[0]);
			//obs.storeInitialisation();
			break;
		}

		World world = null;
		world = World.Instance(observers);
		world.run();
//		System.out.println("Model execution finished properly.");
		return;

	}
}
