//Copyright 2013 Katarzyna Kułakowska, Tomasz Kułakowski.
//
//This file is part of SkyPig.
//
//SkyPig is free software; you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation; either version 2 of the License, or
//(at your option) any later version.
//
//SkyPig is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with SkyPig; if not, write to the Free Software
//Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

package model.core;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashSet;
import java.util.TreeMap;

/**
 *
 * @author kasia
 */
public class Signature {
	/*
	 * Fields:
	 */
	/**
	 *
	 */
	private String tableName;
	/**
	 *
	 */
	private TreeMap<String, String[]> primitiveFields;
	/**
	 *
	 */
	private HashSet<Signature> internalObjects;

	private ArrayList<String> primaryKeys;
	/*
	 * Methods:
	 */

	/**
	 *
	 * @param signatory
	 */
	public Signature(Observable signatory) {
		if (Config.getParameter(signatory.getClass().getSimpleName() + "_tableName") != null) {
			tableName = Config.getParameter(signatory.getClass().getSimpleName() + "_tableName").toString().toLowerCase();
		}
		else {
			tableName =  "default" + signatory.getClass().getSimpleName().toLowerCase();
		}
 		Calendar date = World.getDate();
		primitiveFields = new TreeMap<String, String[]>();
		internalObjects = new HashSet<Signature>();
		primaryKeys = new ArrayList<String>();
		addPrimitiveField("date","date","'"+Integer.toString(date.get(Calendar.YEAR))+"-"+
				Integer.toString(date.get(Calendar.MONTH) +1)+"-"+
				Integer.toString(date.get(Calendar.DAY_OF_MONTH))+"'");
		addPrimitiveField("configurationName","text", "'" + (String) Config.getParameter("configurationName") + "'");
		addPrimitiveField("startTime","text", (String) Config.getParameter("startTime"));
		addPrimitiveField("configurationFile","text", "'" + String.valueOf((String) Config.getParameter("configurationFile")) + "'");
		addPrimitiveField("Signatory","text","'" + signatory.getClass().getSimpleName() + "'");
		addPrimaryKey("startTime");
		addPrimaryKey("configurationFile");
		addPrimaryKey("date");
	}

	/**
	 *
	 * @return
	 */
	public TreeMap<String, String[]> getPrimitiveFields() {
		return primitiveFields;
	}
	
	/**
	 * 
	 * @return
	 */
	public String getNaNField() {
		if (primitiveFields.containsKey("nan")) {
			return primitiveFields.get("nan")[2];
		} else {
			return "No nan fields.";
		}
	}
	
	/**
	 * 
	 * @param field
	 * @return
	 */
	public String getPrimitiveField(String field) {
		return primitiveFields.get(field)[1];
	}

	/**
	 *
	 * @param key
	 * @param type
	 * @param value
	 */
	public void addPrimitiveField(String key, String type, String value) {
		String[] typeAndValue = {type,value};
		primitiveFields.put(key, typeAndValue);
		return;
	}

	/**
	 *
	 * @return
	 */
	public HashSet<Signature> getInternalObjects() {
		return internalObjects;
	}
	
	/**
	 *
	 * @param subSignature
	 */
	public void addInternalObject(Signature subSignature) {
		internalObjects.add(subSignature);
		return;
	}

	/**
	 *
	 * @return
	 */
	public String getTableName() {
		return tableName;
	}
	
	/**
	 * 
	 * @param key
	 */
	public void addPrimaryKey(String key) {
		primaryKeys.add(key);
	}
	
	/**
	 * 
	 * @param newname
	 */
	public void setTableName(String newname) {
		tableName = newname;
	}
	
	/**
	 * 
	 * @return
	 */
	public String getPrimaryKey() {
		String keys = "PRIMARY KEY (";
		for (String a : primaryKeys) {
			keys += a + ", ";
		}
		keys = keys.substring(0, keys.length() - 2);
		return keys + ")";
	}

}
