//Copyright 2013 Katarzyna Kułakowska, Tomasz Kułakowski.
//
//This file is part of SkyPig.
//
//SkyPig is free software; you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation; either version 2 of the License, or
//(at your option) any later version.
//
//SkyPig is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with SkyPig; if not, write to the Free Software
//Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

package model.core;

import java.util.HashMap;

/**
 * Enables creation of proper types of Agents in implementing classes. 
 * 
 * <p>Each species-specific model should implement this interface.</p>
 * 
 * @author Katarzyna Kułakowska, Tomasz Kułakowski
 * @since 2013-01-04
 */
public interface SpeciesFactory {

	/**
	 *	Enables creation of proper instance of Animal in implementing class.
	 * 
	 * @param parameters specific to the constructor of the <code>Animal</code> 
	 * class that is instantiated. 
	 * @return new Animal of the specified class.
	 * @since 2013-01-04
	 */
	Animal createAnimal(HashMap<String, Object> parameters);
	/**
	 *	Enables creation of proper instance of Nest in implementing class.
	 * 
	 * @param parameters specific to the constructor of the <code>Nest</code> 
	 * class that is instantiated. 
	 * @return new Nest of the specified class.
	 * @since 2013-01-04
	 */
	Nest createNest(HashMap<String, Object> parameters);
	/**
	 *	Enables creation of proper instance of LandscapeUnit in implementing class.
	 * 
	 * @param parameters specific to the constructor of the <code>LandscapeUnit</code> 
	 * class that is instantiated. 
	 * @return new LandscapeUnit of the specified class.
	 * @since 2013-01-04
	 */
	LandscapeUnit createLandscapeUnit(HashMap<String, Object> parameters);
}
