//Copyright 2013 Katarzyna Kułakowska, Tomasz Kułakowski.
//
//This file is part of SkyPig.
//
//SkyPig is free software; you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation; either version 2 of the License, or
//(at your option) any later version.
//
//SkyPig is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with SkyPig; if not, write to the Free Software
//Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

package model.core;

import java.sql.SQLException;
import java.util.*;
import java.util.logging.Level;
import java.util.logging.Logger;
import model.core.Animal.Sex;
import model.skylark.SkylarkAnimal;
import model.woodpigeon.WoodpigeonAnimal;

/**
 *
 * @author kasia
 */
public class SpecificObserver extends Observer {

	private ArrayList<String> results;
	private Integer year;
	// for breeding_success analysis
	private int annualNumberOfEggsLaid;
	private int annualNumberOfEggsHatched;
	private int annualNumberOfNestlingsFledged;
	private int annualNumberOfEggsLaidEarly;
	private int annualNumberOfEggsHatchedEarly;
	private int annualNumberOfNestlingsFledgedEarly;
	private int annualNumberOfEggsLaidLate;
	private int annualNumberOfEggsHatchedLate;
	private int annualNumberOfNestlingsFledgedLate;
	private int annualNumberOfFirstYear;
	private int annualNumberOfFemales;
	private int annualInitialNumberOfAdults;
	private int annualFinalNumberOfAdults;
	// for sensitivity analysis
	private Integer timeToExtinction;
	private ArrayList<Double> adultSurvival;
	private ArrayList<Double> juvenileSurvival;
	private ArrayList<Double> nestlingSurvival;
	private ArrayList<Double> nestlingSurvivalEarly;
	private ArrayList<Double> nestlingSurvivalLate;
	private ArrayList<Double> fledglingBodyweight;
	private ArrayList<Double> fledglingBodyweightEarly;
	private ArrayList<Double> fledglingBodyweightLate;
	private ArrayList<Double> fledglingAge;
	private ArrayList<Double> fledglingAgeEarly;
	private ArrayList<Double> fledglingAgeLate;
	private ArrayList<Double> eggSurvivalEarly;
	private ArrayList<Double> eggSurvivalLate;
	private ArrayList<Double> eggSurvival;
	private ArrayList<Double> eggsPerFemale;
	private ArrayList<Double> deathAge;
	private ArrayList<Double> deathBodyweight;
	private ArrayList<Double> renestingIntervals;
	private ArrayList<Double> successfulBroods;
	private ArrayList<Double> allBroods;
	private ArrayList<Double> dailyForagingTimesFeb;
	private ArrayList<Double> fieldsRevisitedFeb;
	private ArrayList<Double> fieldsNumberFeb;
	private ArrayList<Double> dailyForagingTimesApr;
	private ArrayList<Double> fieldsRevisitedApr;
	private ArrayList<Double> fieldsNumberApr;
	private ArrayList<Double> dailyForagingTimesJun;
	private ArrayList<Double> fieldsRevisitedJun;
	private ArrayList<Double> fieldsNumberJun;
	private ArrayList<Double> dailyForagingTimesAug;
	private ArrayList<Double> fieldsRevisitedAug;
	private ArrayList<Double> fieldsNumberAug;
	private ArrayList<Double> dailyForagingTimesOct;
	private ArrayList<Double> fieldsRevisitedOct;
	private ArrayList<Double> fieldsNumberOct;
	private ArrayList<Double> dailyForagingTimesDec;
	private ArrayList<Double> fieldsRevisitedDec;
	private ArrayList<Double> fieldsNumberDec;

	/**
	 * 
	 * @param databaseName
	 */
	public SpecificObserver(String databaseName) {
		super(databaseName);
		results = (ArrayList<String>) Config.getParameter("Observer_resultTypes");
		year = null;

		annualNumberOfEggsLaid = 0;
		annualNumberOfEggsHatched = 0;
		annualNumberOfEggsLaidEarly = 0;
		annualNumberOfEggsHatchedEarly = 0;
		annualNumberOfEggsLaidLate = 0;
		annualNumberOfEggsHatchedLate = 0;
		annualNumberOfNestlingsFledged = 0;
		annualNumberOfNestlingsFledgedEarly = 0;
		annualNumberOfNestlingsFledgedLate = 0;
		annualNumberOfFirstYear = 0;
		annualNumberOfFemales = 0;
		annualInitialNumberOfAdults = 0;
		annualFinalNumberOfAdults = 0;

		timeToExtinction = null;
		adultSurvival = new ArrayList<Double>();
		juvenileSurvival = new ArrayList<Double>();
		nestlingSurvival = new ArrayList<Double>();
		nestlingSurvivalEarly = new ArrayList<Double>();
		nestlingSurvivalLate = new ArrayList<Double>();
		eggSurvival = new ArrayList<Double>();
		eggSurvivalEarly = new ArrayList<Double>();
		eggSurvivalLate = new ArrayList<Double>();

		fledglingBodyweight = new ArrayList<Double>();
		fledglingBodyweightEarly = new ArrayList<Double>();
		fledglingBodyweightLate = new ArrayList<Double>();
		fledglingAge = new ArrayList<Double>();
		fledglingAgeEarly = new ArrayList<Double>();
		fledglingAgeLate = new ArrayList<Double>();
		eggsPerFemale = new ArrayList<Double>();
		deathAge = new ArrayList<Double>();
		deathBodyweight = new ArrayList<Double>();
		renestingIntervals = new ArrayList<Double>();
		successfulBroods = new ArrayList<Double>();
		allBroods = new ArrayList<Double>();
		dailyForagingTimesFeb = new ArrayList<Double>();
		fieldsRevisitedFeb = new ArrayList<Double>();
		fieldsNumberFeb = new ArrayList<Double>();
		dailyForagingTimesApr = new ArrayList<Double>();
		fieldsRevisitedApr = new ArrayList<Double>();
		fieldsNumberApr = new ArrayList<Double>();
		dailyForagingTimesJun = new ArrayList<Double>();
		fieldsRevisitedJun = new ArrayList<Double>();
		fieldsNumberJun = new ArrayList<Double>();
		dailyForagingTimesAug = new ArrayList<Double>();
		fieldsRevisitedAug = new ArrayList<Double>();
		fieldsNumberAug = new ArrayList<Double>();
		dailyForagingTimesOct = new ArrayList<Double>();
		fieldsRevisitedOct = new ArrayList<Double>();
		fieldsNumberOct = new ArrayList<Double>();
		dailyForagingTimesDec = new ArrayList<Double>();
		fieldsRevisitedDec = new ArrayList<Double>();
		fieldsNumberDec = new ArrayList<Double>();

		makeTables();
	}

	@Override
	public void observe(Collection<Observable> observed) {
		try {
			if (results.contains("breeding_success")) {
				if (year != null) {
					if (year != World.getDate().get(Calendar.YEAR)) {
						String update = "INSERT INTO specific_breeding_success (starttime, year, ini_adults, first_year, females, eggs, hatched, fledged, final_adults) VALUES ('";
						update += Config.getParameter("startTime")
								+ "', " + year
								+ ", " + annualInitialNumberOfAdults
								+ ", " + annualNumberOfFirstYear
								+ ", " + annualNumberOfFemales
								+ ", " + annualNumberOfEggsLaid
								+ ", " + annualNumberOfEggsHatched
								+ ", " + annualNumberOfNestlingsFledged
								+ ", " + annualFinalNumberOfAdults + ");";
						addBatch(update);
						annualNumberOfEggsLaid = 0;
						annualNumberOfEggsHatched = 0;
						annualNumberOfNestlingsFledged = 0;
						annualNumberOfFirstYear = 0;
						annualNumberOfFemales = 0;
						annualInitialNumberOfAdults = 0;
						annualFinalNumberOfAdults = 0;
						year = World.getDate().get(Calendar.YEAR);
					}
				} else {
					year = World.getDate().get(Calendar.YEAR);
				}
				for (Observable o : observed) {
					try {
						Animal a = (Animal) o;
						if (World.getDate().get(Calendar.DAY_OF_YEAR)
								== (Integer) Config.getParameter("SeasonalActivity_startOfSeason")) {
							if (a.getDaysSinceHatching() < 365) {
								annualNumberOfFirstYear++;
							}
						}
						if (World.getDate().get(Calendar.DAY_OF_YEAR)
								== (Integer) Config.getParameter("SeasonalActivity_startOfSeason")) {
							annualInitialNumberOfAdults++;
							if (a.getSex() == Sex.FEMALE) {
								annualNumberOfFemales++;
							}
						}
						if (World.getDate().get(Calendar.DAY_OF_YEAR)
								== (Integer) Config.getParameter("SeasonalActivity_endOfSeason") - 1) {
							if (a.getDaysSinceHatching() > 364) {
								annualFinalNumberOfAdults++;
							}
						}
						if (a.getDaysSinceLeavingNest() == 0) {
							annualNumberOfNestlingsFledged++;
						}
					} catch (Exception e1) {
						try {
							Nest n = (Nest) o;
							takeNest(n);
						} catch (Exception e2) {
							// DO NOTHING
						}
					}
				}
			}
			if (results.contains("debug")) {
				super.observe(observed);
			}
			if (results.contains("sensitivity")) {
				if (year != null) {
					saveDailyRecords(observed);
					checkTimeToExtinction();
					saveAnnualBreedingSuccess();
					saveAnnualSpringRecords(observed);
					resetAnnualFields();
				} else {
					year = World.getDate().get(Calendar.YEAR);
				}
			}
			executeBatch();
		} catch (SQLException e) {
			Logger.getLogger(Observer.class.getName()).log(Level.SEVERE, null,
					e);
			Logger.getLogger(Observer.class.getName()).log(Level.SEVERE, null,
					e.getNextException());
		}
	}

	private void saveDailyRecords(Collection<Observable> observed) {
		for (Observable o : observed) {
			try {
				Animal a = (Animal) o;
				takeAnimal(a);
				recordFeeding(a);
			} catch (Exception e1) {
				try {
					Nest n = (Nest) o;
					takeNest(n);
				} catch (Exception e2) {
					// DO NOTHING
				}
			}
		}
	}

	private void recordFeeding(Animal a) {
		try {
			WoodpigeonAnimal animal = (WoodpigeonAnimal) a;
			if ((World.getDate().get(Calendar.MONTH) == 1
					|| World.getDate().get(Calendar.MONTH) == 3
					|| World.getDate().get(Calendar.MONTH) == 5
					|| World.getDate().get(Calendar.MONTH) == 7
					|| World.getDate().get(Calendar.MONTH) == 9
					|| World.getDate().get(Calendar.MONTH) == 11)
					&& Config.getGenerator().nextDouble() < 0.01
					&& !Boolean.valueOf(animal.getSignature().getPrimitiveField("migration"))
					&& World.getDate().get(Calendar.YEAR)
					> ((GregorianCalendar) Config.getParameter("World_startDate")).get(Calendar.YEAR)) {
				Signature s = animal.getSignature();
				HashSet<Signature> subSigs = s.getInternalObjects();
				String timeForagedToday = null;
				String revisitedFields = null;
				String numberOfFields = null;
				Iterator<Signature> it = subSigs.iterator();
				while (timeForagedToday == null && it.hasNext()) {
					Signature stateSig = it.next();
					if (stateSig.getPrimitiveField("Signatory").contains("Adult")) {
						timeForagedToday = stateSig.getPrimitiveField("timeForagedToday");
						if (!stateSig.getPrimitiveField("revisitedFields").equalsIgnoreCase("null")) {
							revisitedFields = stateSig.getPrimitiveField("revisitedFields");
						}
						numberOfFields = stateSig.getPrimitiveField("numberOfFields");
					}
				}
				if (timeForagedToday != null) {
					if (World.getDate().get(Calendar.MONTH) == 1) {
						dailyForagingTimesFeb.add(Double.valueOf(timeForagedToday));
						fieldsRevisitedFeb.add(Double.valueOf(revisitedFields));
						fieldsNumberFeb.add(Double.valueOf(numberOfFields));
					} else if (World.getDate().get(Calendar.MONTH) == 3) {
						dailyForagingTimesApr.add(Double.valueOf(timeForagedToday));
						fieldsRevisitedApr.add(Double.valueOf(revisitedFields));
						fieldsNumberApr.add(Double.valueOf(numberOfFields));
					} else if (World.getDate().get(Calendar.MONTH) == 5) {
						dailyForagingTimesJun.add(Double.valueOf(timeForagedToday));
						fieldsRevisitedJun.add(Double.valueOf(revisitedFields));
						fieldsNumberJun.add(Double.valueOf(numberOfFields));
					} else if (World.getDate().get(Calendar.MONTH) == 7) {
						dailyForagingTimesAug.add(Double.valueOf(timeForagedToday));
						fieldsRevisitedAug.add(Double.valueOf(revisitedFields));
						fieldsNumberAug.add(Double.valueOf(numberOfFields));
					} else if (World.getDate().get(Calendar.MONTH) == 9) {
						dailyForagingTimesOct.add(Double.valueOf(timeForagedToday));
						fieldsRevisitedOct.add(Double.valueOf(revisitedFields));
						fieldsNumberOct.add(Double.valueOf(numberOfFields));
					} else if (World.getDate().get(Calendar.MONTH) == 11) {
						dailyForagingTimesDec.add(Double.valueOf(timeForagedToday));
						fieldsRevisitedDec.add(Double.valueOf(revisitedFields));
						fieldsNumberDec.add(Double.valueOf(numberOfFields));
					}
				}
			}
		} catch (Exception e1) {
		}
	}

	private void takeNest(Nest n) {
		if (n.getDaysSinceLaying() == 0) {
			annualNumberOfEggsLaid += n.getOffspringSize();
			if (n.getDateOfLaying().get(Calendar.DAY_OF_YEAR)
					< (Integer) Config.getParameter("SpecificObserver_earlyAndLateBroods")) {
				annualNumberOfEggsLaidEarly += n.getOffspringSize();
			} else {
				annualNumberOfEggsLaidLate += n.getOffspringSize();
			}
		}
		if (n.getDaysSinceHatching() == 0) {
			annualNumberOfEggsHatched += n.getOffspringSize();
			if (n.getDateOfLaying().get(Calendar.DAY_OF_YEAR)
					< (Integer) Config.getParameter("SpecificObserver_earlyAndLateBroods")) {
				annualNumberOfEggsHatchedEarly += n.getOffspringSize();
			} else {
				annualNumberOfEggsHatchedLate += n.getOffspringSize();
			}
		}
		if (annualNumberOfEggsHatchedEarly > annualNumberOfEggsLaidEarly) {
			int ala = 0;
		}
	}

	private void takeAnimal(Animal a) {
		if (a.getDaysSinceLeavingNest() == 0) {
			annualNumberOfNestlingsFledged += 1;
			fledglingBodyweight.add(new Double(a.getBodyweight()));
			fledglingAge.add(new Double(a.getDaysSinceHatching()));
			if (a.getDateOfLaying().get(Calendar.DAY_OF_YEAR)
					< (Integer) Config.getParameter("SpecificObserver_earlyAndLateBroods")) {
				annualNumberOfNestlingsFledgedEarly++;
				fledglingBodyweightEarly.add(new Double(a.getBodyweight()));
				fledglingAgeEarly.add(new Double(a.getDaysSinceHatching()));
			} else {
				annualNumberOfNestlingsFledgedLate++;
				fledglingBodyweightLate.add(new Double(a.getBodyweight()));
				fledglingAgeLate.add(new Double(a.getDaysSinceHatching()));
			}
		} else if (a.getDaysSinceLeavingNest() > 0 && a.getAlive() == false) {
			deathAge.add(new Double(a.getDaysSinceLaying()) / 365.0);
			deathBodyweight.add(a.getBodyweight());
			if (a.getSex() == Sex.FEMALE && a.getAllBroods() > 0
					&& World.getDate().get(Calendar.YEAR)
					> ((GregorianCalendar) Config.getParameter("World_startDate")).get(Calendar.YEAR)) {
				successfulBroods.add(new Double(a.getSuccessfulBroods()));
				allBroods.add(new Double(a.getAllBroods()));
			}
		}
		if (a.getSex() == Sex.FEMALE) {
			takeRenestingInterval(a);
		}
	}

	private void takeRenestingInterval(Animal a) {
		Signature s = a.getSignature();
		HashSet<Signature> subSigs = s.getInternalObjects();
		String renesting = null;
		Iterator<Signature> it = subSigs.iterator();
		while (renesting == null && it.hasNext()) {
			Signature stateSig = it.next();
			if (stateSig.getPrimitiveField("Signatory").contains("Adult")) {
				renesting = stateSig.getPrimitiveField("renesting");
			}
		}
		if (!"null".equalsIgnoreCase(renesting)) {
			renestingIntervals.add(Double.valueOf(renesting));
		}
	}

	private Integer checkTimeToExtinction() {
		if (new Integer(World.Instance().getSignature().getPrimitiveField("numberOfAdults")) == 0
				&& timeToExtinction == null) {
			timeToExtinction = World.getDate().get(Calendar.YEAR)
					- ((GregorianCalendar) Config.getParameter("World_startDate")).get(Calendar.YEAR);
		}
		return timeToExtinction;
	}

	private void saveAnnualSpringRecords(Collection<Observable> observed) {
		if (World.getDate().get(Calendar.DAY_OF_YEAR)
				== (Integer) Config.getParameter("SeasonalActivity_startOfSeason")) {
			int finalNumberOfAdults = 0;
			int finalNumberOfJuveniles = 0;
			Double annualEggsPerFemale = null;
			if (annualNumberOfFemales > 0) {
				annualEggsPerFemale = new Double(annualNumberOfEggsLaid) / new Double(annualNumberOfFemales);
			}
			annualNumberOfFemales = 0;
			for (Observable o : observed) {
				try {
					Animal a = (Animal) o;
					if (a.getDaysSinceLaying() > 365) {
						finalNumberOfAdults++;
					} else {
						finalNumberOfJuveniles++;
					}
					if (a.getSex() == Sex.FEMALE && a.getAlive() == true) {
						annualNumberOfFemales++;
						if (a.getAllBroods() > 0
								&& World.getDate().get(Calendar.YEAR)
								> ((GregorianCalendar) Config.getParameter("World_startDate")).get(Calendar.YEAR)) {
							successfulBroods.add(new Double(a.getSuccessfulBroods()));
							allBroods.add(new Double(a.getAllBroods()));
						}
					}
				} catch (Exception ex) {
				}
			}
			if (World.getDate().get(Calendar.YEAR)
					> ((GregorianCalendar) Config.getParameter("World_startDate")).get(Calendar.YEAR)) {
				if (annualInitialNumberOfAdults > 0) {
					adultSurvival.add(new Double(finalNumberOfAdults) / new Double(annualInitialNumberOfAdults));
				}
				if (annualNumberOfNestlingsFledged > 0) {
					juvenileSurvival.add(new Double(finalNumberOfJuveniles) / new Double(annualNumberOfNestlingsFledged));
				}
				if (annualEggsPerFemale != null) {
					eggsPerFemale.add(annualEggsPerFemale);
				}
			}
			annualInitialNumberOfAdults = finalNumberOfAdults + finalNumberOfJuveniles;
		}
	}

	private void saveAnnualBreedingSuccess() {
		if (World.getDate().get(Calendar.DAY_OF_YEAR)
				== (Integer) Config.getParameter("SeasonalActivity_startOfSeason")
				&& World.getDate().get(Calendar.YEAR)
				> ((GregorianCalendar) Config.getParameter("World_startDate")).get(Calendar.YEAR)) {
			if (annualNumberOfEggsLaid > 0) {
				eggSurvival.add(new Double(annualNumberOfEggsHatched) / new Double(annualNumberOfEggsLaid));
			}
			if (annualNumberOfEggsHatched > 0) {
				nestlingSurvival.add(new Double(annualNumberOfNestlingsFledged) / new Double(annualNumberOfEggsHatched));
			}
			if (annualNumberOfEggsLaidEarly > 0) {
				eggSurvivalEarly.add(new Double(annualNumberOfEggsHatchedEarly) / new Double(annualNumberOfEggsLaidEarly));
			}
			if (annualNumberOfEggsHatchedEarly > 0) {
				nestlingSurvivalEarly.add(new Double(annualNumberOfNestlingsFledgedEarly) / new Double(annualNumberOfEggsHatchedEarly));
			}
			if (annualNumberOfEggsLaidLate > 0) {
				eggSurvivalLate.add(new Double(annualNumberOfEggsHatchedLate) / new Double(annualNumberOfEggsLaidLate));
			}
			if (annualNumberOfEggsHatchedLate > 0) {
				nestlingSurvivalLate.add(new Double(annualNumberOfNestlingsFledgedLate) / new Double(annualNumberOfEggsHatchedLate));
			}
		}
	}

	private void resetAnnualFields() {
		if (World.getDate().get(Calendar.DAY_OF_YEAR)
				== (Integer) Config.getParameter("SeasonalActivity_startOfSeason")) {
			annualNumberOfEggsLaid = 0;
			annualNumberOfEggsHatched = 0;
			annualNumberOfEggsLaidEarly = 0;
			annualNumberOfEggsHatchedEarly = 0;
			annualNumberOfEggsLaidLate = 0;
			annualNumberOfEggsHatchedLate = 0;
			annualNumberOfNestlingsFledged = 0;
			annualNumberOfNestlingsFledgedEarly = 0;
			annualNumberOfNestlingsFledgedLate = 0;
			annualNumberOfFirstYear = 0;
		}
	}

	@Override
	public void shutdown() {
		finallyObserve();
		super.shutdown();
	}

	private void makeTables() {
		try {
			if (!getTables("specific_breeding_success").next()) {
				executeStatement("CREATE TABLE specific_breeding_success (starttime text, year integer, ini_adults integer, first_year integer, females integer, eggs integer, hatched integer, fledged integer, final_adults integer, PRIMARY KEY (starttime, year));");
			}
			if (!getTables("sensitivity").next()) {
				executeStatement("CREATE TABLE IF NOT EXISTS sensitivity (starttime text, "
						+ "configurationFile text"
						+ "parameter_Adult_deathProbability1 double precision, "
						+ "parameter_Adult_deathProbability2 double precision, "
						+ "parameter_Animal_bmrDeathProbability double precision, "
						+ "parameter_Animal_juvenileDesiredSurvivalTime double precision, "
						+ "parameter_Animal_desiredSurvivalTime double precision, "
						+ "parameter_Animal_desiredSurvivalProbability double precision, "
						+ "parameter_WoodpigeonAdult_daysOfHungerBeforeMigration integer, "
						+ "parameter_WoodpigeonAnimal_imigrationProbability double precision, "
						+ "parameter_Reproduce_readyForReproduction double precision, "
						+ "parameter_WoodpigeonAdult_feedingRoutineLength integer, "
						+ "parameter_Reproduce_reproductionBuffer double precision, "
						+ "parameter_WoodpigeonAdult_memoryLength integer, "
						+ "parameter_WoodpigeonAdult_proportionOfBetterFields double precision, "
						+ "parameter_Config_timeFactor integer, "
						+ "parameter_WoodpigeonLandscapeUnit_numberOfDisturbances integer, "
						+ "parameter_Eggs_predationRate double precision, "
						+ "parameter_Eggs_backgroundPredationRate double precision, "
						+ "parameter_Nestlings_predationRate double precision, "
						+ "parameter_SeasonalActivity_startOfSeason integer, "
						+ "parameter_SeasonalActivity_endOfSeason integer, "
						+ "parameter_SeasonalActivity_lateStartOfSeason integer, "
						+ "result_extinction_time integer, "
						+ "result_avg_adults_survival double precision, "
						+ "result_min_adults_survival double precision, "
						+ "result_max_adults_survival double precision, "
						+ "result_sd_adults_survival double precision, "
						+ "result_se_adults_survival double precision, "
						+ "result_median_adults_survival double precision, "
						+ "result_25perc_adults_survival double precision, "
						+ "result_75perc_adults_survival double precision, "
						+ "result_avg_juveniles_survival double precision, "
						+ "result_min_juveniles_survival double precision, "
						+ "result_max_juveniles_survival double precision, "
						+ "result_sd_juveniles_survival double precision, "
						+ "result_se_juveniles_survival double precision, "
						+ "result_median_juveniles_survival double precision, "
						+ "result_25perc_juveniles_survival double precision, "
						+ "result_75perc_juveniles_survival double precision, "
						+ "result_avg_eggs_survival double precision, "
						+ "result_min_eggs_survival double precision, "
						+ "result_max_eggs_survival double precision, "
						+ "result_sd_eggs_survival double precision, "
						+ "result_se_eggs_survival double precision, "
						+ "result_median_eggs_survival double precision, "
						+ "result_25perc_eggs_survival double precision, "
						+ "result_75perc_eggs_survival double precision, "
						+ "result_avg_nestlings_survival double precision, "
						+ "result_min_nestlings_survival double precision, "
						+ "result_max_nestlings_survival double precision, "
						+ "result_sd_nestlings_survival double precision, "
						+ "result_se_nestlings_survival double precision, "
						+ "result_median_nestlings_survival double precision, "
						+ "result_25perc_nestlings_survival double precision, "
						+ "result_75perc_nestlings_survival double precision, "
						+ "result_avg_eggs_per_female double precision, "
						+ "result_min_eggs_per_female double precision, "
						+ "result_max_eggs_per_female double precision, "
						+ "result_sd_eggs_per_female double precision, "
						+ "result_se_eggs_per_female double precision, "
						+ "result_median_eggs_per_female double precision, "
						+ "result_25perc_eggs_per_female double precision, "
						+ "result_75perc_eggs_per_female double precision, "
						+ "result_avg_fledglings_bodyweight double precision, "
						+ "result_min_fledglings_bodyweight double precision, "
						+ "result_max_fledglings_bodyweight double precision, "
						+ "result_sd_fledglings_bodyweight double precision, "
						+ "result_se_fledglings_bodyweight double precision, "
						+ "result_median_fledglings_bodyweight double precision, "
						+ "result_25perc_fledglings_bodyweight double precision, "
						+ "result_75perc_fledglings_bodyweight double precision, "
						+ "result_avg_fledglings_age double precision, "
						+ "result_min_fledglings_age double precision, "
						+ "result_max_fledglings_age double precision, "
						+ "result_sd_fledglings_age double precision, "
						+ "result_se_fledglings_age double precision, "
						+ "result_median_fledglings_age double precision, "
						+ "result_25perc_fledglings_age double precision, "
						+ "result_75perc_fledglings_age double precision, "
						+ "result_avg_eggs_survival_early double precision, "
						+ "result_avg_eggs_survival_late double precision, "
						+ "result_avg_nestlings_survival_early double precision, "
						+ "result_avg_nestlings_survival_late double precision, "
						+ "result_avg_fledglings_bodyweight_late double precision, "
						+ "result_avg_fledglings_bodyweight_early double precision, "
						+ "result_avg_fledglings_age_late double precision, "
						+ "result_avg_fledglings_age_early double precision, "
						+ "result_proportion_fledglings_early double precision, "
						+ "result_number_fledglings double precision, "
						+ "result_avg_dead_bodyweight double precision, "
						+ "result_min_dead_bodyweight double precision, "
						+ "result_max_dead_bodyweight double precision, "
						+ "result_sd_dead_bodyweight double precision, "
						+ "result_se_dead_bodyweight double precision, "
						+ "result_median_dead_bodyweight double precision, "
						+ "result_25perc_dead_bodyweight double precision, "
						+ "result_75perc_dead_bodyweight double precision, "
						+ "result_median_perc_dead_age double precision, "
						+ "result_95perc_dead_age double precision, "
						+ "result_number_dead double precision, "
						+ "result_0successful_broods double precision, "
						+ "result_1successful_broods double precision, "
						+ "result_2successful_broods double precision, "
						+ "result_3successful_broods double precision, "
						+ "result_4successful_broods double precision, "
						+ "result_5successful_broods double precision, "
						+ "result_avg_successful_broods double precision, "
						+ "result_avg_all_broods double precision, "
						+ "result_number_broods double precision, "
						+ "result_avg_renesting_interval double precision, "
						+ "result_min_renesting_interval double precision, "
						+ "result_max_renesting_interval double precision, "
						+ "result_sd_renesting_interval double precision, "
						+ "result_se_renesting_interval double precision, "
						+ "result_median_renesting_interval double precision, "
						+ "result_25perc_renesting_interval double precision, "
						+ "result_75perc_renesting_interval double precision, "
						+ "result_number_renesting_interval double precision, "
						+ "result_avg_daily_foraging_times_feb double precision, "
						+ "result_sd_daily_foraging_times_feb double precision, "
						+ "result_se_daily_foraging_times_feb double precision, "
						+ "result_median_daily_foraging_times_feb double precision, "
						+ "result_25perc_daily_foraging_times_feb double precision, "
						+ "result_75perc_daily_foraging_times_feb double precision, "
						+ "result_max_daily_foraging_times_feb double precision, "
						+ "result_min_daily_foraging_times_feb double precision, "
						+ "result_avg_daily_foraging_times_apr double precision, "
						+ "result_sd_daily_foraging_times_apr double precision, "
						+ "result_se_daily_foraging_times_apr double precision, "
						+ "result_median_daily_foraging_times_apr double precision, "
						+ "result_25perc_daily_foraging_times_apr double precision, "
						+ "result_75perc_daily_foraging_times_apr double precision, "
						+ "result_max_daily_foraging_times_apr double precision, "
						+ "result_min_daily_foraging_times_apr double precision, "
						+ "result_avg_daily_foraging_times_jun double precision, "
						+ "result_sd_daily_foraging_times_jun double precision, "
						+ "result_se_daily_foraging_times_jun double precision, "
						+ "result_median_daily_foraging_times_jun double precision, "
						+ "result_25perc_daily_foraging_times_jun double precision, "
						+ "result_75perc_daily_foraging_times_jun double precision, "
						+ "result_max_daily_foraging_times_jun double precision, "
						+ "result_min_daily_foraging_times_jun double precision, "
						+ "result_avg_daily_foraging_times_aug double precision, "
						+ "result_sd_daily_foraging_times_aug double precision, "
						+ "result_se_daily_foraging_times_aug double precision, "
						+ "result_median_daily_foraging_times_aug double precision, "
						+ "result_25perc_daily_foraging_times_aug double precision, "
						+ "result_75perc_daily_foraging_times_aug double precision, "
						+ "result_max_daily_foraging_times_aug double precision, "
						+ "result_min_daily_foraging_times_aug double precision, "
						+ "result_avg_daily_foraging_times_oct double precision, "
						+ "result_sd_daily_foraging_times_oct double precision, "
						+ "result_se_daily_foraging_times_oct double precision, "
						+ "result_median_daily_foraging_times_oct double precision, "
						+ "result_25perc_daily_foraging_times_oct double precision, "
						+ "result_75perc_daily_foraging_times_oct double precision, "
						+ "result_max_daily_foraging_times_oct double precision, "
						+ "result_min_daily_foraging_times_oct double precision, "
						+ "result_avg_daily_foraging_times_dec double precision, "
						+ "result_sd_daily_foraging_times_dec double precision, "
						+ "result_se_daily_foraging_times_dec double precision, "
						+ "result_median_daily_foraging_times_dec double precision, "
						+ "result_25perc_daily_foraging_times_dec double precision, "
						+ "result_75perc_daily_foraging_times_dec double precision, "
						+ "result_max_daily_foraging_times_dec double precision, "
						+ "result_min_daily_foraging_times_dec double precision, "
						+ "result_avg_number_of_fields_feb double precision, "
						+ "result_sd_number_of_fields_feb double precision, "
						+ "result_se_number_of_fields_feb double precision, "
						+ "result_median_number_of_fields_feb double precision, "
						+ "result_25perc_number_of_fields_feb double precision, "
						+ "result_75perc_number_of_fields_feb double precision, "
						+ "result_max_number_of_fields_feb double precision, "
						+ "result_min_number_of_fields_feb double precision, "
						+ "result_avg_number_of_fields_apr double precision, "
						+ "result_sd_number_of_fields_apr double precision, "
						+ "result_se_number_of_fields_apr double precision, "
						+ "result_median_number_of_fields_apr double precision, "
						+ "result_25perc_number_of_fields_apr double precision, "
						+ "result_75perc_number_of_fields_apr double precision, "
						+ "result_max_number_of_fields_apr double precision, "
						+ "result_min_number_of_fields_apr double precision, "
						+ "result_avg_number_of_fields_jun double precision, "
						+ "result_sd_number_of_fields_jun double precision, "
						+ "result_se_number_of_fields_jun double precision, "
						+ "result_median_number_of_fields_jun double precision, "
						+ "result_25perc_number_of_fields_jun double precision, "
						+ "result_75perc_number_of_fields_jun double precision, "
						+ "result_max_number_of_fields_jun double precision, "
						+ "result_min_number_of_fields_jun double precision, "
						+ "result_avg_number_of_fields_aug double precision, "
						+ "result_sd_number_of_fields_aug double precision, "
						+ "result_se_number_of_fields_aug double precision, "
						+ "result_median_number_of_fields_aug double precision, "
						+ "result_25perc_number_of_fields_aug double precision, "
						+ "result_75perc_number_of_fields_aug double precision, "
						+ "result_max_number_of_fields_aug double precision, "
						+ "result_min_number_of_fields_aug double precision, "
						+ "result_avg_number_of_fields_oct double precision, "
						+ "result_sd_number_of_fields_oct double precision, "
						+ "result_se_number_of_fields_oct double precision, "
						+ "result_median_number_of_fields_oct double precision, "
						+ "result_25perc_number_of_fields_oct double precision, "
						+ "result_75perc_number_of_fields_oct double precision, "
						+ "result_max_number_of_fields_oct double precision, "
						+ "result_min_number_of_fields_oct double precision, "
						+ "result_avg_number_of_fields_dec double precision, "
						+ "result_sd_number_of_fields_dec double precision, "
						+ "result_se_number_of_fields_dec double precision, "
						+ "result_median_number_of_fields_dec double precision, "
						+ "result_25perc_number_of_fields_dec double precision, "
						+ "result_75perc_number_of_fields_dec double precision, "
						+ "result_max_number_of_fields_dec double precision, "
						+ "result_min_number_of_fields_dec double precision, "
						+ "result_avg_fields_revisited_feb double precision, "
						+ "result_sd_fields_revisited_feb double precision, "
						+ "result_se_fields_revisited_feb double precision, "
						+ "result_median_fields_revisited_feb double precision, "
						+ "result_25perc_fields_revisited_feb double precision, "
						+ "result_75perc_fields_revisited_feb double precision, "
						+ "result_proportion_all_fields_revisited_feb double precision, "
						+ "result_proportion_none_fields_revisited_feb double precision, "
						+ "result_avg_fields_revisited_apr double precision, "
						+ "result_sd_fields_revisited_apr double precision, "
						+ "result_se_fields_revisited_apr double precision, "
						+ "result_median_fields_revisited_apr double precision, "
						+ "result_25perc_fields_revisited_apr double precision, "
						+ "result_75perc_fields_revisited_apr double precision, "
						+ "result_proportion_all_fields_revisited_apr double precision, "
						+ "result_proportion_none_fields_revisited_apr double precision, "
						+ "result_avg_fields_revisited_jun double precision, "
						+ "result_sd_fields_revisited_jun double precision, "
						+ "result_se_fields_revisited_jun double precision, "
						+ "result_median_fields_revisited_jun double precision, "
						+ "result_25perc_fields_revisited_jun double precision, "
						+ "result_75perc_fields_revisited_jun double precision, "
						+ "result_proportion_all_fields_revisited_jun double precision, "
						+ "result_proportion_none_fields_revisited_jun double precision, "
						+ "result_avg_fields_revisited_aug double precision, "
						+ "result_sd_fields_revisited_aug double precision, "
						+ "result_se_fields_revisited_aug double precision, "
						+ "result_median_fields_revisited_aug double precision, "
						+ "result_25perc_fields_revisited_aug double precision, "
						+ "result_75perc_fields_revisited_aug double precision, "
						+ "result_proportion_all_fields_revisited_aug double precision, "
						+ "result_proportion_none_fields_revisited_aug double precision, "
						+ "result_avg_fields_revisited_oct double precision, "
						+ "result_sd_fields_revisited_oct double precision, "
						+ "result_se_fields_revisited_oct double precision, "
						+ "result_median_fields_revisited_oct double precision, "
						+ "result_25perc_fields_revisited_oct double precision, "
						+ "result_75perc_fields_revisited_oct double precision, "
						+ "result_proportion_all_fields_revisited_oct double precision, "
						+ "result_proportion_none_fields_revisited_oct double precision, "
						+ "result_avg_fields_revisited_dec double precision, "
						+ "result_sd_fields_revisited_dec double precision, "
						+ "result_se_fields_revisited_dec double precision, "
						+ "result_median_fields_revisited_dec double precision, "
						+ "result_25perc_fields_revisited_dec double precision, "
						+ "result_75perc_fields_revisited_dec double precision, "
						+ "result_proportion_all_fields_revisited_dec double precision, "
						+ "result_proportion_none_fields_revisited_dec double precision, "
						+ "PRIMARY KEY (starttime));");
			}
		} catch (SQLException e) {
			e.printStackTrace();
			e.getNextException().printStackTrace();
		}
	}

	/**
	 * 
	 */
	public void finallyObserve() {
		if (results.contains("sensitivity")) {
			try {
				String update = "INSERT INTO sensitivity (starttime, "
						+ "configurationFile, "
						+ "parameter_Adult_deathProbability1, "
						+ "parameter_Adult_deathProbability2, "
						+ "parameter_Animal_bmrDeathProbability, "
						+ "parameter_Animal_juvenileDesiredSurvivalTime, "
						+ "parameter_Animal_desiredSurvivalTime, "
						+ "parameter_Animal_desiredSurvivalProbability, "
						+ "parameter_WoodpigeonAdult_daysOfHungerBeforeMigration, "
						+ "parameter_WoodpigeonAnimal_imigrationProbability, "
						+ "parameter_Reproduce_readyForReproduction, "
						+ "parameter_WoodpigeonAdult_feedingRoutineLength, "
						+ "parameter_Reproduce_reproductionBuffer, "
						+ "parameter_WoodpigeonAdult_memoryLength, "
						+ "parameter_WoodpigeonAdult_proportionOfBetterFields, "
						+ "parameter_Config_timeFactor, "
						+ "parameter_WoodpigeonLandscapeUnit_numberOfDisturbances, "
						+ "parameter_Eggs_predationRate, "
						+ "parameter_Eggs_backgroundPredationRate, "
						+ "parameter_Nestlings_predationRate, "
						+ "parameter_SeasonalActivity_startOfSeason, "
						+ "parameter_SeasonalActivity_endOfSeason, "
						+ "parameter_SeasonalActivity_lateStartOfSeason, "
						+ "result_extinction_time, "
						+ "result_avg_adults_survival, "
						+ "result_min_adults_survival, "
						+ "result_max_adults_survival, "
						+ "result_sd_adults_survival, "
						+ "result_se_adults_survival, "
						+ "result_median_adults_survival, "
						+ "result_25perc_adults_survival, "
						+ "result_75perc_adults_survival, "
						+ "result_avg_juveniles_survival, "
						+ "result_min_juveniles_survival, "
						+ "result_max_juveniles_survival, "
						+ "result_sd_juveniles_survival, "
						+ "result_se_juveniles_survival, "
						+ "result_median_juveniles_survival, "
						+ "result_25perc_juveniles_survival, "
						+ "result_75perc_juveniles_survival, "
						+ "result_avg_eggs_survival, "
						+ "result_min_eggs_survival, "
						+ "result_max_eggs_survival, "
						+ "result_sd_eggs_survival, "
						+ "result_se_eggs_survival, "
						+ "result_median_eggs_survival, "
						+ "result_25perc_eggs_survival, "
						+ "result_75perc_eggs_survival, "
						+ "result_avg_nestlings_survival, "
						+ "result_min_nestlings_survival, "
						+ "result_max_nestlings_survival, "
						+ "result_sd_nestlings_survival, "
						+ "result_se_nestlings_survival, "
						+ "result_median_nestlings_survival, "
						+ "result_25perc_nestlings_survival, "
						+ "result_75perc_nestlings_survival, "
						+ "result_avg_eggs_per_female, "
						+ "result_min_eggs_per_female, "
						+ "result_max_eggs_per_female, "
						+ "result_sd_eggs_per_female, "
						+ "result_se_eggs_per_female, "
						+ "result_median_eggs_per_female, "
						+ "result_25perc_eggs_per_female, "
						+ "result_75perc_eggs_per_female, "
						+ "result_avg_fledglings_bodyweight, "
						+ "result_min_fledglings_bodyweight, "
						+ "result_max_fledglings_bodyweight, "
						+ "result_sd_fledglings_bodyweight, "
						+ "result_se_fledglings_bodyweight, "
						+ "result_median_fledglings_bodyweight, "
						+ "result_25perc_fledglings_bodyweight, "
						+ "result_75perc_fledglings_bodyweight, "
						+ "result_avg_fledglings_age, "
						+ "result_min_fledglings_age, "
						+ "result_max_fledglings_age, "
						+ "result_sd_fledglings_age, "
						+ "result_se_fledglings_age, "
						+ "result_median_fledglings_age, "
						+ "result_25perc_fledglings_age, "
						+ "result_75perc_fledglings_age, "
						+ "result_avg_eggs_survival_early, "
						+ "result_avg_eggs_survival_late, "
						+ "result_avg_nestlings_survival_early, "
						+ "result_avg_nestlings_survival_late, "
						+ "result_avg_fledglings_bodyweight_late, "
						+ "result_avg_fledglings_bodyweight_early, "
						+ "result_avg_fledglings_age_late, "
						+ "result_avg_fledglings_age_early, "
						+ "result_proportion_fledglings_early, "
						+ "result_number_fledglings, "
						+ "result_avg_dead_bodyweight, "
						+ "result_min_dead_bodyweight, "
						+ "result_max_dead_bodyweight, "
						+ "result_sd_dead_bodyweight, "
						+ "result_se_dead_bodyweight, "
						+ "result_median_dead_bodyweight, "
						+ "result_25perc_dead_bodyweight, "
						+ "result_75perc_dead_bodyweight, "
						+ "result_median_perc_dead_age, "
						+ "result_95perc_dead_age, "
						+ "result_number_dead, "
						+ "result_0successful_broods, "
						+ "result_1successful_broods, "
						+ "result_2successful_broods, "
						+ "result_3successful_broods, "
						+ "result_4successful_broods, "
						+ "result_5successful_broods, "
						+ "result_avg_successful_broods, "
						+ "result_avg_all_broods, "
						+ "result_number_broods, "
						+ "result_avg_renesting_interval, "
						+ "result_min_renesting_interval, "
						+ "result_max_renesting_interval, "
						+ "result_sd_renesting_interval, "
						+ "result_se_renesting_interval, "
						+ "result_median_renesting_interval, "
						+ "result_25perc_renesting_interval, "
						+ "result_75perc_renesting_interval, "
						+ "result_number_renesting_interval, "
						+ "result_avg_daily_foraging_times_feb, "
						+ "result_sd_daily_foraging_times_feb, "
						+ "result_se_daily_foraging_times_feb, "
						+ "result_median_daily_foraging_times_feb, "
						+ "result_25perc_daily_foraging_times_feb, "
						+ "result_75perc_daily_foraging_times_feb, "
						+ "result_max_daily_foraging_times_feb, "
						+ "result_min_daily_foraging_times_feb, "
						+ "result_avg_daily_foraging_times_apr, "
						+ "result_sd_daily_foraging_times_apr, "
						+ "result_se_daily_foraging_times_apr, "
						+ "result_median_daily_foraging_times_apr, "
						+ "result_25perc_daily_foraging_times_apr, "
						+ "result_75perc_daily_foraging_times_apr, "
						+ "result_max_daily_foraging_times_apr, "
						+ "result_min_daily_foraging_times_apr, "
						+ "result_avg_daily_foraging_times_jun, "
						+ "result_sd_daily_foraging_times_jun, "
						+ "result_se_daily_foraging_times_jun, "
						+ "result_median_daily_foraging_times_jun, "
						+ "result_25perc_daily_foraging_times_jun, "
						+ "result_75perc_daily_foraging_times_jun, "
						+ "result_max_daily_foraging_times_jun, "
						+ "result_min_daily_foraging_times_jun, "
						+ "result_avg_daily_foraging_times_aug, "
						+ "result_sd_daily_foraging_times_aug, "
						+ "result_se_daily_foraging_times_aug, "
						+ "result_median_daily_foraging_times_aug, "
						+ "result_25perc_daily_foraging_times_aug, "
						+ "result_75perc_daily_foraging_times_aug, "
						+ "result_max_daily_foraging_times_aug, "
						+ "result_min_daily_foraging_times_aug, "
						+ "result_avg_daily_foraging_times_oct, "
						+ "result_sd_daily_foraging_times_oct, "
						+ "result_se_daily_foraging_times_oct, "
						+ "result_median_daily_foraging_times_oct, "
						+ "result_25perc_daily_foraging_times_oct, "
						+ "result_75perc_daily_foraging_times_oct, "
						+ "result_max_daily_foraging_times_oct, "
						+ "result_min_daily_foraging_times_oct, "
						+ "result_avg_daily_foraging_times_dec, "
						+ "result_sd_daily_foraging_times_dec, "
						+ "result_se_daily_foraging_times_dec, "
						+ "result_median_daily_foraging_times_dec, "
						+ "result_25perc_daily_foraging_times_dec, "
						+ "result_75perc_daily_foraging_times_dec, "
						+ "result_max_daily_foraging_times_dec, "
						+ "result_min_daily_foraging_times_dec, "
						+ "result_avg_number_of_fields_feb, "
						+ "result_sd_number_of_fields_feb, "
						+ "result_se_number_of_fields_feb, "
						+ "result_median_number_of_fields_feb, "
						+ "result_25perc_number_of_fields_feb, "
						+ "result_75perc_number_of_fields_feb, "
						+ "result_max_number_of_fields_feb, "
						+ "result_min_number_of_fields_feb, "
						+ "result_avg_number_of_fields_apr, "
						+ "result_sd_number_of_fields_apr, "
						+ "result_se_number_of_fields_apr, "
						+ "result_median_number_of_fields_apr, "
						+ "result_25perc_number_of_fields_apr, "
						+ "result_75perc_number_of_fields_apr, "
						+ "result_max_number_of_fields_apr, "
						+ "result_min_number_of_fields_apr, "
						+ "result_avg_number_of_fields_jun, "
						+ "result_sd_number_of_fields_jun, "
						+ "result_se_number_of_fields_jun, "
						+ "result_median_number_of_fields_jun, "
						+ "result_25perc_number_of_fields_jun, "
						+ "result_75perc_number_of_fields_jun, "
						+ "result_max_number_of_fields_jun, "
						+ "result_min_number_of_fields_jun, "
						+ "result_avg_number_of_fields_aug, "
						+ "result_sd_number_of_fields_aug, "
						+ "result_se_number_of_fields_aug, "
						+ "result_median_number_of_fields_aug, "
						+ "result_25perc_number_of_fields_aug, "
						+ "result_75perc_number_of_fields_aug, "
						+ "result_max_number_of_fields_aug, "
						+ "result_min_number_of_fields_aug, "
						+ "result_avg_number_of_fields_oct, "
						+ "result_sd_number_of_fields_oct, "
						+ "result_se_number_of_fields_oct, "
						+ "result_median_number_of_fields_oct, "
						+ "result_25perc_number_of_fields_oct, "
						+ "result_75perc_number_of_fields_oct, "
						+ "result_max_number_of_fields_oct, "
						+ "result_min_number_of_fields_oct, "
						+ "result_avg_number_of_fields_dec, "
						+ "result_sd_number_of_fields_dec, "
						+ "result_se_number_of_fields_dec, "
						+ "result_median_number_of_fields_dec, "
						+ "result_25perc_number_of_fields_dec, "
						+ "result_75perc_number_of_fields_dec, "
						+ "result_max_number_of_fields_dec, "
						+ "result_min_number_of_fields_dec, "
						+ "result_avg_fields_revisited_feb, "
						+ "result_sd_fields_revisited_feb, "
						+ "result_se_fields_revisited_feb, "
						+ "result_median_fields_revisited_feb, "
						+ "result_25perc_fields_revisited_feb, "
						+ "result_75perc_fields_revisited_feb, "
						+ "result_proportion_all_fields_revisited_feb, "
						+ "result_proportion_none_fields_revisited_feb, "
						+ "result_avg_fields_revisited_apr, "
						+ "result_sd_fields_revisited_apr, "
						+ "result_se_fields_revisited_apr, "
						+ "result_median_fields_revisited_apr, "
						+ "result_25perc_fields_revisited_apr, "
						+ "result_75perc_fields_revisited_apr, "
						+ "result_proportion_all_fields_revisited_apr, "
						+ "result_proportion_none_fields_revisited_apr, "
						+ "result_avg_fields_revisited_jun, "
						+ "result_sd_fields_revisited_jun, "
						+ "result_se_fields_revisited_jun, "
						+ "result_median_fields_revisited_jun, "
						+ "result_25perc_fields_revisited_jun, "
						+ "result_75perc_fields_revisited_jun, "
						+ "result_proportion_all_fields_revisited_jun, "
						+ "result_proportion_none_fields_revisited_jun, "
						+ "result_avg_fields_revisited_aug, "
						+ "result_sd_fields_revisited_aug, "
						+ "result_se_fields_revisited_aug, "
						+ "result_median_fields_revisited_aug, "
						+ "result_25perc_fields_revisited_aug, "
						+ "result_75perc_fields_revisited_aug, "
						+ "result_proportion_all_fields_revisited_aug, "
						+ "result_proportion_none_fields_revisited_aug, "
						+ "result_avg_fields_revisited_oct, "
						+ "result_sd_fields_revisited_oct, "
						+ "result_se_fields_revisited_oct, "
						+ "result_median_fields_revisited_oct, "
						+ "result_25perc_fields_revisited_oct, "
						+ "result_75perc_fields_revisited_oct, "
						+ "result_proportion_all_fields_revisited_oct, "
						+ "result_proportion_none_fields_revisited_oct, "
						+ "result_avg_fields_revisited_dec, "
						+ "result_sd_fields_revisited_dec, "
						+ "result_se_fields_revisited_dec, "
						+ "result_median_fields_revisited_dec, "
						+ "result_25perc_fields_revisited_dec, "
						+ "result_75perc_fields_revisited_dec, "
						+ "result_proportion_all_fields_revisited_dec, "
						+ "result_proportion_none_fields_revisited_dec"
						+ ") VALUES ('";
				update += Config.getParameter("startTime")
						+ "', " + "'" + Config.getParameter("configurationFile") + "'"
						+ ", " + Config.getParameter("Adult_deathProbability1")
						+ ", " + Config.getParameter("Adult_deathProbability2")
						+ ", " + Config.getParameter("Animal_bmrDeathProbability")
						+ ", " + Config.getParameter("Animal_juvenileDesiredSurvivalTime")
						+ ", " + Config.getParameter("Animal_desiredSurvivalTime")
						+ ", " + Config.getParameter("Animal_desiredSurvivalProbability")
						+ ", " + Config.getParameter("WoodpigeonAdult_daysOfHungerBeforeMigration")
						+ ", " + Config.getParameter("WoodpigeonAnimal_imigrationProbability")
						+ ", " + Config.getParameter("Reproduce_readyForReproduction")
						+ ", " + Config.getParameter("WoodpigeonAdult_feedingRoutineLength")
						+ ", " + Config.getParameter("Reproduce_reproductionBuffer")
						+ ", " + Config.getParameter("WoodpigeonAdult_memoryLength")
						+ ", " + Config.getParameter("WoodpigeonAdult_proportionOfBetterFields")
						+ ", " + Config.getParameter("Config_timeFactor")
						+ ", " + Config.getParameter("WoodpigeonLandscapeUnit_numberOfDisturbances")
						+ ", " + Config.getParameter("Eggs_predationRate")
						+ ", " + Config.getParameter("Eggs_backgroundPredationRate")
						+ ", " + Config.getParameter("Nestlings_predationRate")
						+ ", " + Config.getParameter("SeasonalActivity_startOfSeason")
						+ ", " + Config.getParameter("SeasonalActivity_endOfSeason")
						+ ", " + Config.getParameter("SeasonalActivity_lateStartOfSeason")
						+ ", " + prepareUpdate(checkTimeToExtinction())
						+ ", " + prepareUpdate(avg(adultSurvival))
						+ ", " + prepareUpdate(min(adultSurvival))
						+ ", " + prepareUpdate(max(adultSurvival))
						+ ", " + prepareUpdate(sdev(adultSurvival))
						+ ", " + prepareUpdate(serr(adultSurvival))
						+ ", " + prepareUpdate(percentile(adultSurvival, 50))
						+ ", " + prepareUpdate(percentile(adultSurvival, 25))
						+ ", " + prepareUpdate(percentile(adultSurvival, 75))
						+ ", " + prepareUpdate(avg(juvenileSurvival))
						+ ", " + prepareUpdate(min(juvenileSurvival))
						+ ", " + prepareUpdate(max(juvenileSurvival))
						+ ", " + prepareUpdate(sdev(juvenileSurvival))
						+ ", " + prepareUpdate(serr(juvenileSurvival))
						+ ", " + prepareUpdate(percentile(juvenileSurvival, 50))
						+ ", " + prepareUpdate(percentile(juvenileSurvival, 25))
						+ ", " + prepareUpdate(percentile(juvenileSurvival, 75))
						+ ", " + prepareUpdate(avg(eggSurvival))
						+ ", " + prepareUpdate(min(eggSurvival))
						+ ", " + prepareUpdate(max(eggSurvival))
						+ ", " + prepareUpdate(sdev(eggSurvival))
						+ ", " + prepareUpdate(serr(eggSurvival))
						+ ", " + prepareUpdate(percentile(eggSurvival, 50))
						+ ", " + prepareUpdate(percentile(eggSurvival, 25))
						+ ", " + prepareUpdate(percentile(eggSurvival, 75))
						+ ", " + prepareUpdate(avg(nestlingSurvival))
						+ ", " + prepareUpdate(min(nestlingSurvival))
						+ ", " + prepareUpdate(max(nestlingSurvival))
						+ ", " + prepareUpdate(sdev(nestlingSurvival))
						+ ", " + prepareUpdate(serr(nestlingSurvival))
						+ ", " + prepareUpdate(percentile(nestlingSurvival, 50))
						+ ", " + prepareUpdate(percentile(nestlingSurvival, 25))
						+ ", " + prepareUpdate(percentile(nestlingSurvival, 75))
						+ ", " + prepareUpdate(avg(eggsPerFemale))
						+ ", " + prepareUpdate(min(eggsPerFemale))
						+ ", " + prepareUpdate(max(eggsPerFemale))
						+ ", " + prepareUpdate(sdev(eggsPerFemale))
						+ ", " + prepareUpdate(serr(eggsPerFemale))
						+ ", " + prepareUpdate(percentile(eggsPerFemale, 50))
						+ ", " + prepareUpdate(percentile(eggsPerFemale, 25))
						+ ", " + prepareUpdate(percentile(eggsPerFemale, 75))
						+ ", " + prepareUpdate(avg(fledglingBodyweight))
						+ ", " + prepareUpdate(min(fledglingBodyweight))
						+ ", " + prepareUpdate(max(fledglingBodyweight))
						+ ", " + prepareUpdate(sdev(fledglingBodyweight))
						+ ", " + prepareUpdate(serr(fledglingBodyweight))
						+ ", " + prepareUpdate(percentile(fledglingBodyweight, 50))
						+ ", " + prepareUpdate(percentile(fledglingBodyweight, 25))
						+ ", " + prepareUpdate(percentile(fledglingBodyweight, 75))
						+ ", " + prepareUpdate(avg(fledglingAge))
						+ ", " + prepareUpdate(min(fledglingAge))
						+ ", " + prepareUpdate(max(fledglingAge))
						+ ", " + prepareUpdate(sdev(fledglingAge))
						+ ", " + prepareUpdate(serr(fledglingAge))
						+ ", " + prepareUpdate(percentile(fledglingAge, 50))
						+ ", " + prepareUpdate(percentile(fledglingAge, 25))
						+ ", " + prepareUpdate(percentile(fledglingAge, 75))
						+ ", " + prepareUpdate(avg(eggSurvivalEarly))
						+ ", " + prepareUpdate(avg(eggSurvivalLate))
						+ ", " + prepareUpdate(avg(nestlingSurvivalEarly))
						+ ", " + prepareUpdate(avg(nestlingSurvivalLate))
						+ ", " + prepareUpdate(avg(fledglingBodyweightLate))
						+ ", " + prepareUpdate(avg(fledglingBodyweightEarly))
						+ ", " + prepareUpdate(avg(fledglingAgeLate))
						+ ", " + prepareUpdate(avg(fledglingAgeEarly))
						+ ", " + prepareUpdate(ratio(fledglingAgeEarly.size(), fledglingAge.size()))
						+ ", " + prepareUpdate(new Integer(fledglingAge.size()))
						+ ", " + prepareUpdate(avg(deathBodyweight))
						+ ", " + prepareUpdate(min(deathBodyweight))
						+ ", " + prepareUpdate(max(deathBodyweight))
						+ ", " + prepareUpdate(sdev(deathBodyweight))
						+ ", " + prepareUpdate(serr(deathBodyweight))
						+ ", " + prepareUpdate(percentile(deathBodyweight, 50))
						+ ", " + prepareUpdate(percentile(deathBodyweight, 25))
						+ ", " + prepareUpdate(percentile(deathBodyweight, 75))
						+ ", " + prepareUpdate(percentile(deathAge, 50))
						+ ", " + prepareUpdate(percentile(deathAge, 95))
						+ ", " + prepareUpdate(new Integer(deathAge.size()))
						+ ", " + prepareUpdate(proportionOfValues(successfulBroods, 0.0))
						+ ", " + prepareUpdate(proportionOfValues(successfulBroods, 1.0))
						+ ", " + prepareUpdate(proportionOfValues(successfulBroods, 2.0))
						+ ", " + prepareUpdate(proportionOfValues(successfulBroods, 3.0))
						+ ", " + prepareUpdate(proportionOfValues(successfulBroods, 4.0))
						+ ", " + prepareUpdate(proportionOfValues(successfulBroods, 5.0))
						+ ", " + prepareUpdate(avg(successfulBroods))
						+ ", " + prepareUpdate(avg(allBroods))
						+ ", " + prepareUpdate(new Integer(allBroods.size()))
						+ ", " + prepareUpdate(avg(renestingIntervals))
						+ ", " + prepareUpdate(min(renestingIntervals))
						+ ", " + prepareUpdate(max(renestingIntervals))
						+ ", " + prepareUpdate(sdev(renestingIntervals))
						+ ", " + prepareUpdate(serr(renestingIntervals))
						+ ", " + prepareUpdate(percentile(renestingIntervals, 50))
						+ ", " + prepareUpdate(percentile(renestingIntervals, 25))
						+ ", " + prepareUpdate(percentile(renestingIntervals, 75))
						+ ", " + prepareUpdate(new Integer(renestingIntervals.size()))
						+ ", " + prepareUpdate(avg(prepareResult(dailyForagingTimesFeb)))
						+ ", " + prepareUpdate(sdev(prepareResult(dailyForagingTimesFeb)))
						+ ", " + prepareUpdate(serr(prepareResult(dailyForagingTimesFeb)))
						+ ", " + prepareUpdate(percentile(prepareResult(dailyForagingTimesFeb), 50))
						+ ", " + prepareUpdate(percentile(prepareResult(dailyForagingTimesFeb), 25))
						+ ", " + prepareUpdate(percentile(prepareResult(dailyForagingTimesFeb), 75))
						+ ", " + prepareUpdate(max(prepareResult(dailyForagingTimesFeb)))
						+ ", " + prepareUpdate(min(prepareResult(dailyForagingTimesFeb)))
						+ ", " + prepareUpdate(avg(prepareResult(dailyForagingTimesApr)))
						+ ", " + prepareUpdate(sdev(prepareResult(dailyForagingTimesApr)))
						+ ", " + prepareUpdate(serr(prepareResult(dailyForagingTimesApr)))
						+ ", " + prepareUpdate(percentile(prepareResult(dailyForagingTimesApr), 50))
						+ ", " + prepareUpdate(percentile(prepareResult(dailyForagingTimesApr), 25))
						+ ", " + prepareUpdate(percentile(prepareResult(dailyForagingTimesApr), 75))
						+ ", " + prepareUpdate(max(prepareResult(dailyForagingTimesApr)))
						+ ", " + prepareUpdate(min(prepareResult(dailyForagingTimesApr)))
						+ ", " + prepareUpdate(avg(prepareResult(dailyForagingTimesJun)))
						+ ", " + prepareUpdate(sdev(prepareResult(dailyForagingTimesJun)))
						+ ", " + prepareUpdate(serr(prepareResult(dailyForagingTimesJun)))
						+ ", " + prepareUpdate(percentile(prepareResult(dailyForagingTimesJun), 50))
						+ ", " + prepareUpdate(percentile(prepareResult(dailyForagingTimesJun), 25))
						+ ", " + prepareUpdate(percentile(prepareResult(dailyForagingTimesJun), 75))
						+ ", " + prepareUpdate(max(prepareResult(dailyForagingTimesJun)))
						+ ", " + prepareUpdate(min(prepareResult(dailyForagingTimesJun)))
						+ ", " + prepareUpdate(avg(prepareResult(dailyForagingTimesAug)))
						+ ", " + prepareUpdate(sdev(prepareResult(dailyForagingTimesAug)))
						+ ", " + prepareUpdate(serr(prepareResult(dailyForagingTimesAug)))
						+ ", " + prepareUpdate(percentile(prepareResult(dailyForagingTimesAug), 50))
						+ ", " + prepareUpdate(percentile(prepareResult(dailyForagingTimesAug), 25))
						+ ", " + prepareUpdate(percentile(prepareResult(dailyForagingTimesAug), 75))
						+ ", " + prepareUpdate(max(prepareResult(dailyForagingTimesAug)))
						+ ", " + prepareUpdate(min(prepareResult(dailyForagingTimesAug)))
						+ ", " + prepareUpdate(avg(prepareResult(dailyForagingTimesOct)))
						+ ", " + prepareUpdate(sdev(prepareResult(dailyForagingTimesOct)))
						+ ", " + prepareUpdate(serr(prepareResult(dailyForagingTimesOct)))
						+ ", " + prepareUpdate(percentile(prepareResult(dailyForagingTimesOct), 50))
						+ ", " + prepareUpdate(percentile(prepareResult(dailyForagingTimesOct), 25))
						+ ", " + prepareUpdate(percentile(prepareResult(dailyForagingTimesOct), 75))
						+ ", " + prepareUpdate(max(prepareResult(dailyForagingTimesOct)))
						+ ", " + prepareUpdate(min(prepareResult(dailyForagingTimesOct)))
						+ ", " + prepareUpdate(avg(prepareResult(dailyForagingTimesDec)))
						+ ", " + prepareUpdate(sdev(prepareResult(dailyForagingTimesDec)))
						+ ", " + prepareUpdate(serr(prepareResult(dailyForagingTimesDec)))
						+ ", " + prepareUpdate(percentile(prepareResult(dailyForagingTimesDec), 50))
						+ ", " + prepareUpdate(percentile(prepareResult(dailyForagingTimesDec), 25))
						+ ", " + prepareUpdate(percentile(prepareResult(dailyForagingTimesDec), 75))
						+ ", " + prepareUpdate(max(prepareResult(dailyForagingTimesDec)))
						+ ", " + prepareUpdate(min(prepareResult(dailyForagingTimesDec)))
						+ ", " + prepareUpdate(avg(prepareResult(fieldsNumberFeb)))
						+ ", " + prepareUpdate(sdev(prepareResult(fieldsNumberFeb)))
						+ ", " + prepareUpdate(serr(prepareResult(fieldsNumberFeb)))
						+ ", " + prepareUpdate(percentile(prepareResult(fieldsNumberFeb), 50))
						+ ", " + prepareUpdate(percentile(prepareResult(fieldsNumberFeb), 25))
						+ ", " + prepareUpdate(percentile(prepareResult(fieldsNumberFeb), 75))
						+ ", " + prepareUpdate(max(prepareResult(fieldsNumberFeb)))
						+ ", " + prepareUpdate(min(prepareResult(fieldsNumberFeb)))
						+ ", " + prepareUpdate(avg(prepareResult(fieldsNumberApr)))
						+ ", " + prepareUpdate(sdev(prepareResult(fieldsNumberApr)))
						+ ", " + prepareUpdate(serr(prepareResult(fieldsNumberApr)))
						+ ", " + prepareUpdate(percentile(prepareResult(fieldsNumberApr), 50))
						+ ", " + prepareUpdate(percentile(prepareResult(fieldsNumberApr), 25))
						+ ", " + prepareUpdate(percentile(prepareResult(fieldsNumberApr), 75))
						+ ", " + prepareUpdate(max(prepareResult(fieldsNumberApr)))
						+ ", " + prepareUpdate(min(prepareResult(fieldsNumberApr)))
						+ ", " + prepareUpdate(avg(prepareResult(fieldsNumberJun)))
						+ ", " + prepareUpdate(sdev(prepareResult(fieldsNumberJun)))
						+ ", " + prepareUpdate(serr(prepareResult(fieldsNumberJun)))
						+ ", " + prepareUpdate(percentile(prepareResult(fieldsNumberJun), 50))
						+ ", " + prepareUpdate(percentile(prepareResult(fieldsNumberJun), 25))
						+ ", " + prepareUpdate(percentile(prepareResult(fieldsNumberJun), 75))
						+ ", " + prepareUpdate(max(prepareResult(fieldsNumberJun)))
						+ ", " + prepareUpdate(min(prepareResult(fieldsNumberJun)))
						+ ", " + prepareUpdate(avg(prepareResult(fieldsNumberAug)))
						+ ", " + prepareUpdate(sdev(prepareResult(fieldsNumberAug)))
						+ ", " + prepareUpdate(serr(prepareResult(fieldsNumberAug)))
						+ ", " + prepareUpdate(percentile(prepareResult(fieldsNumberAug), 50))
						+ ", " + prepareUpdate(percentile(prepareResult(fieldsNumberAug), 25))
						+ ", " + prepareUpdate(percentile(prepareResult(fieldsNumberAug), 75))
						+ ", " + prepareUpdate(max(prepareResult(fieldsNumberAug)))
						+ ", " + prepareUpdate(min(prepareResult(fieldsNumberAug)))
						+ ", " + prepareUpdate(avg(prepareResult(fieldsNumberOct)))
						+ ", " + prepareUpdate(sdev(prepareResult(fieldsNumberOct)))
						+ ", " + prepareUpdate(serr(prepareResult(fieldsNumberOct)))
						+ ", " + prepareUpdate(percentile(prepareResult(fieldsNumberOct), 50))
						+ ", " + prepareUpdate(percentile(prepareResult(fieldsNumberOct), 25))
						+ ", " + prepareUpdate(percentile(prepareResult(fieldsNumberOct), 75))
						+ ", " + prepareUpdate(max(prepareResult(fieldsNumberOct)))
						+ ", " + prepareUpdate(min(prepareResult(fieldsNumberOct)))
						+ ", " + prepareUpdate(avg(prepareResult(fieldsNumberDec)))
						+ ", " + prepareUpdate(sdev(prepareResult(fieldsNumberDec)))
						+ ", " + prepareUpdate(serr(prepareResult(fieldsNumberDec)))
						+ ", " + prepareUpdate(percentile(prepareResult(fieldsNumberDec), 50))
						+ ", " + prepareUpdate(percentile(prepareResult(fieldsNumberDec), 25))
						+ ", " + prepareUpdate(percentile(prepareResult(fieldsNumberDec), 75))
						+ ", " + prepareUpdate(max(prepareResult(fieldsNumberDec)))
						+ ", " + prepareUpdate(min(prepareResult(fieldsNumberDec)))
						+ ", " + prepareUpdate(avg(prepareResult(fieldsRevisitedFeb)))
						+ ", " + prepareUpdate(sdev(prepareResult(fieldsRevisitedFeb)))
						+ ", " + prepareUpdate(serr(prepareResult(fieldsRevisitedFeb)))
						+ ", " + prepareUpdate(percentile(prepareResult(fieldsRevisitedFeb), 50))
						+ ", " + prepareUpdate(percentile(prepareResult(fieldsRevisitedFeb), 25))
						+ ", " + prepareUpdate(percentile(prepareResult(fieldsRevisitedFeb), 75))
						+ ", " + prepareUpdate(proportionOfValues(prepareResult(fieldsRevisitedFeb), 1))
						+ ", " + prepareUpdate(proportionOfValues(prepareResult(fieldsRevisitedFeb), 0))
						+ ", " + prepareUpdate(avg(prepareResult(fieldsRevisitedApr)))
						+ ", " + prepareUpdate(sdev(prepareResult(fieldsRevisitedApr)))
						+ ", " + prepareUpdate(serr(prepareResult(fieldsRevisitedApr)))
						+ ", " + prepareUpdate(percentile(prepareResult(fieldsRevisitedApr), 50))
						+ ", " + prepareUpdate(percentile(prepareResult(fieldsRevisitedApr), 25))
						+ ", " + prepareUpdate(percentile(prepareResult(fieldsRevisitedApr), 75))
						+ ", " + prepareUpdate(proportionOfValues(prepareResult(fieldsRevisitedApr), 1))
						+ ", " + prepareUpdate(proportionOfValues(prepareResult(fieldsRevisitedApr), 0))
						+ ", " + prepareUpdate(avg(prepareResult(fieldsRevisitedJun)))
						+ ", " + prepareUpdate(sdev(prepareResult(fieldsRevisitedJun)))
						+ ", " + prepareUpdate(serr(prepareResult(fieldsRevisitedJun)))
						+ ", " + prepareUpdate(percentile(prepareResult(fieldsRevisitedJun), 50))
						+ ", " + prepareUpdate(percentile(prepareResult(fieldsRevisitedJun), 25))
						+ ", " + prepareUpdate(percentile(prepareResult(fieldsRevisitedJun), 75))
						+ ", " + prepareUpdate(proportionOfValues(prepareResult(fieldsRevisitedJun), 1))
						+ ", " + prepareUpdate(proportionOfValues(prepareResult(fieldsRevisitedJun), 0))
						+ ", " + prepareUpdate(avg(prepareResult(fieldsRevisitedAug)))
						+ ", " + prepareUpdate(sdev(prepareResult(fieldsRevisitedAug)))
						+ ", " + prepareUpdate(serr(prepareResult(fieldsRevisitedAug)))
						+ ", " + prepareUpdate(percentile(prepareResult(fieldsRevisitedAug), 50))
						+ ", " + prepareUpdate(percentile(prepareResult(fieldsRevisitedAug), 25))
						+ ", " + prepareUpdate(percentile(prepareResult(fieldsRevisitedAug), 75))
						+ ", " + prepareUpdate(proportionOfValues(prepareResult(fieldsRevisitedAug), 1))
						+ ", " + prepareUpdate(proportionOfValues(prepareResult(fieldsRevisitedAug), 0))
						+ ", " + prepareUpdate(avg(prepareResult(fieldsRevisitedOct)))
						+ ", " + prepareUpdate(sdev(prepareResult(fieldsRevisitedOct)))
						+ ", " + prepareUpdate(serr(prepareResult(fieldsRevisitedOct)))
						+ ", " + prepareUpdate(percentile(prepareResult(fieldsRevisitedOct), 50))
						+ ", " + prepareUpdate(percentile(prepareResult(fieldsRevisitedOct), 25))
						+ ", " + prepareUpdate(percentile(prepareResult(fieldsRevisitedOct), 75))
						+ ", " + prepareUpdate(proportionOfValues(prepareResult(fieldsRevisitedOct), 1))
						+ ", " + prepareUpdate(proportionOfValues(prepareResult(fieldsRevisitedOct), 0))
						+ ", " + prepareUpdate(avg(prepareResult(fieldsRevisitedDec)))
						+ ", " + prepareUpdate(sdev(prepareResult(fieldsRevisitedDec)))
						+ ", " + prepareUpdate(serr(prepareResult(fieldsRevisitedDec)))
						+ ", " + prepareUpdate(percentile(prepareResult(fieldsRevisitedDec), 50))
						+ ", " + prepareUpdate(percentile(prepareResult(fieldsRevisitedDec), 25))
						+ ", " + prepareUpdate(percentile(prepareResult(fieldsRevisitedDec), 75))
						+ ", " + prepareUpdate(proportionOfValues(prepareResult(fieldsRevisitedDec), 1))
						+ ", " + prepareUpdate(proportionOfValues(prepareResult(fieldsRevisitedDec), 0))
						+ ");";
				addBatch(update);
				executeBatch();
			} catch (SQLException ex) {
				Logger.getLogger(SpecificObserver.class.getName()).log(Level.SEVERE, null, ex);
			}
		}
	}

}
