/*
 //Copyright 2013 Katarzyna Kułakowska, Tomasz Kułakowski.
 //
 //This file is part of SkyPig.
 //
 //SkyPig is free software; you can redistribute it and/or modify
 //it under the terms of the GNU General Public License as published by
 //the Free Software Foundation; either version 2 of the License, or
 //(at your option) any later version.
 //
 //SkyPig is distributed in the hope that it will be useful,
 //but WITHOUT ANY WARRANTY; without even the implied warranty of
 //MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 //GNU General Public License for more details.
 //
 //You should have received a copy of the GNU General Public License
 //along with SkyPig; if not, write to the Free Software
 //Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package model.woodpigeon;

import java.sql.SQLException;
import java.util.*;
import java.util.logging.Level;
import java.util.logging.Logger;
import model.core.*;
import model.core.Observable;
import model.core.Observer;

/**
 *
 * @author kasia
 */
public class DisturbanceObserver extends Observer {

	ArrayList<String> results;
	private double annualMaxNumberOfNests;
	private double annualNumberOfNestlingsFledged;
	private double annualNumberOfEggsLaid;
	private double annualNumberOfEggsHatched;
	private boolean shooting;
	private boolean nestDestruction;
	private double dailyShootingProbability;
	private double annualShootingProbability;
	private double nestDestructionProbability;
	private boolean pesticide;
	private double pesticideSlope;
	private double initialPopulationSize;
	private double pesticideMortality;
	private ArrayList<Double> lethalDoses;

	public DisturbanceObserver(String databaseName) {
		super(databaseName);
		results = (ArrayList<String>) Config.getParameter("Observer_resultTypes");
		lethalDoses = new ArrayList<Double>();
		makeTable();
		reset();
		//to avoid errors in the first save day
		annualMaxNumberOfNests = 1;
		initialPopulationSize = 1.0;
	}

	public void observe(Collection<Observable> observed) {
		if (results.contains("debug")) {
			super.observe(observed);
		}
		int nests = World.Instance().getNumberOfAdultsAndNests()[1];
		if (nests > annualMaxNumberOfNests) {
			annualMaxNumberOfNests = nests;
		}
		if (World.getDate().get(Calendar.DAY_OF_YEAR) == (Integer) Config.getParameter("Spring_cereals_sowingDayStart")) {
			initialPopulationSize = World.Instance().getNumberOfAdultsAndNests()[0];
		}
		for (Observable o : observed) {
			try {
				Animal a = (Animal) o;
				if (a.getDaysSinceLeavingNest() == 0) {
					annualNumberOfNestlingsFledged++;
				}
				if (((a.getSignature()).getPrimitiveField("causeOfDeath")).equalsIgnoreCase("'mortalityEffect'")) {
					pesticideMortality += 1;
					for (String pest : (ArrayList<String>) Config.getParameter("Pesticides")) {
						lethalDoses.add(1000.0 * (a.getPesticides()).get(pest) / a.getBodyweight());
					}
				}

			} catch (Exception e1) {
				try {
					Nest n = (Nest) o;
					if (n.getDaysSinceLaying() == 0) {
						annualNumberOfEggsLaid += n.getOffspringSize();
					}
					if (n.getDaysSinceHatching() == 0) {
						annualNumberOfEggsHatched += n.getOffspringSize();
					}
				} catch (Exception e2) {
					// DO NOTHING
				}
			}
		}
		if (World.getDate().get(Calendar.DAY_OF_YEAR)
				== (Integer) Config.getParameter("DisturbanceObserver_annualSaveDay")) {
			//nest destruction?
			if ((Boolean) Config.getParameter("WoodpigeonNest_destruction")) {
				for (Calendar date : (ArrayList<GregorianCalendar>) Config.getParameter("WoodpigeonNest_destructionDates")) {
					if (World.getDate().get(Calendar.YEAR) == date.get(Calendar.YEAR)) {
						nestDestruction = true;
						nestDestructionProbability = (Double) Config.getParameter("WoodpigeonNest_destructionProbability");
					}
				}
			}
			//shooting?
			if ((Boolean) Config.getParameter("WoodpigeonAnimal_shooting")) {
				ArrayList<Integer> years = (ArrayList<Integer>) Config.getParameter("WoodpigeonAnimal_shootingYears");
				for (Integer year : years) {
					if (World.getDate().get(Calendar.YEAR) == year) {
						int yearIndex = years.indexOf(year);
						shooting = true;
						dailyShootingProbability = ((ArrayList<Double>) Config.getParameter("WoodpigeonAnimal_shootingProbability")).get(yearIndex);
						annualShootingProbability = dailyShootingProbability
								* ((Integer) Config.getParameter("WoodpigeonAnimal_shootingEndDay")
								- (Integer) Config.getParameter("WoodpigeonAnimal_shootingStartDay") + 1);
					}
				}
			}
			//pesticide?
			if (Config.getParameter("Pesticides") != null) {
				HashMap<String, Object> copy = Config.getParameterList();
				for (String pest : (ArrayList<String>) Config.getParameter("Pesticides")) {
					for (String par : copy.keySet()) {
						if (par.endsWith("_pesticide") && copy.get(par) != null) {
							if (((HashMap<String, HashMap<String, Object>>) copy.get(par)).containsKey(pest)) {
								if (Config.checkIfPesticideApplicationYear(pest, "Spring_cereals")) {
									pesticide = true;
									pesticideSlope = (Double) Config.getParameter(pest + "_mortalityEffect_maxdose_slope");
								}
							}
						}
					}
				}
			}
//				if (Config.getParameter("Pesticides") != null && World.getDate().get(GregorianCalendar.YEAR)
//						== ((Integer) Config.getParameter("WoodpigeonLandscapeUnit_pesticideYear"))) {
//					pesticide = true;
//					for (String pest : (ArrayList<String>) Config.getParameter("Pesticides")) {
//						pesticideSlope = (Double) Config.getParameter(pest + "_mortalityEffect_maxdose_slope");
//					}
//				}
			//pop_size
			try {
				String update = "INSERT INTO  disturbance_effects ("
						+ "starttime, "
						+ "year, "
						+ "pop_size, "
						+ "nest_destruction, "
						+ "nest_destruction_probability, "
						+ "shooting, "
						+ "dailyShooting_probability, "
						+ "annualShooting_probability, "
						+ "pesticide, "
						+ "pesticide_slope, "
						+ "pesticide_mortality, "
						+ "lethal_doses, "
						+ "fledged, "
						+ "mnos, "
						+ "laid, "
						+ "hatched)"
						+ " VALUES ('"
						+ Config.getParameter("startTime")
						+ "', " + World.getDate().get(Calendar.YEAR)
						+ ", " + prepareUpdate(initialPopulationSize)
						+ ", " + prepareUpdate(nestDestruction)
						+ ", " + prepareUpdate(nestDestructionProbability)
						+ ", " + prepareUpdate(shooting)
						+ ", " + prepareUpdate(dailyShootingProbability)
						+ ", " + prepareUpdate(annualShootingProbability)
						+ ", " + prepareUpdate(pesticide)
						+ ", " + prepareUpdate(pesticideSlope)
						+ ", " + prepareUpdate(pesticideMortality / initialPopulationSize)
						+ ", " + prepareUpdate(lethalDoses)
						+ ", " + prepareUpdate(annualNumberOfNestlingsFledged)
						+ ", " + prepareUpdate(annualMaxNumberOfNests)
						+ ", " + prepareUpdate(annualNumberOfEggsLaid)
						+ ", " + prepareUpdate(annualNumberOfEggsHatched)
						+ ");";
				addBatch(update);
				executeBatch();
			} catch (SQLException ex) {
				Logger.getLogger(SpecificObserver.class.getName()).log(Level.SEVERE, null, ex);
			}
			reset();
		}
	}

	private void reset() {
		annualNumberOfNestlingsFledged = 0;
		annualMaxNumberOfNests = 0;
		annualNumberOfEggsLaid = 0;
		annualNumberOfEggsHatched = 0;
		shooting = false;
		nestDestruction = false;
		dailyShootingProbability = 0.0;
		annualShootingProbability = 0.0;
		nestDestructionProbability = 0.0;
		pesticide = false;
		pesticideSlope = 0;
		initialPopulationSize = 0.0;
		pesticideMortality = 0;
		lethalDoses.clear();
	}

	public void makeTable() {
		try {
			executeStatement("CREATE TABLE IF NOT EXISTS disturbance_effects ("
					+ "starttime text, "
					+ "year integer, "
					+ "pop_size integer, "
					+ "nest_destruction boolean, "
					+ "nest_destruction_probability double precision, "
					+ "shooting boolean, "
					+ "dailyShooting_probability double precision, "
					+ "annualShooting_probability double precision, "
					+ "pesticide boolean, "
					+ "pesticide_slope double precision, "
					+ "pesticide_mortality double precision, "
					+ "lethal_doses double precision[], "
					+ "fledged double precision, "
					+ "mnos double precision, "
					+ "laid double precision, "
					+ "hatched double precision, "
					+ "PRIMARY KEY (starttime, year));");
		} catch (SQLException e) {
			e.printStackTrace();
			e.getNextException().printStackTrace();
		}
	}
}
