//Copyright 2013 Katarzyna Kułakowska, Tomasz Kułakowski.
//
//This file is part of SkyPig.
//
//SkyPig is free software; you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation; either version 2 of the License, or
//(at your option) any later version.
//
//SkyPig is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with SkyPig; if not, write to the Free Software
//Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
package model.woodpigeon;

import java.sql.SQLException;
import java.util.*;
import java.util.logging.Level;
import java.util.logging.Logger;
import model.core.Observable;
import model.core.Observer;
import model.core.*;

/**
 *
 * @author kasia
 */
public class SensitivityObserver extends Observer {

	ArrayList<String> results;
	String pattern;
	private double mnos_ref;
	private double fledged_ref;
	private double laid_ref;
	private double hatched_ref;
	private double mnos_exp;
	private double fledged_exp;
	private double laid_exp;
	private double hatched_exp;
	private double annualShootingProbability;
	private double pesticideSlope;
	private double mortality;
	private double initialPopulationSize;
	private int year;
	private TreeMap<String, TreeMap<Integer, SortedSet<Double>>> foragingDistribution;
	private ArrayList<Double> numberOfFields;
	private ArrayList<Double> revisitedFields;
	private ArrayList<Double> numberOfFields1000;
	private ArrayList<Double> revisitedFields1000;
	private ArrayList<Double> breedingDensity;
	private ArrayList<Double> popSize;
	private double nestingArea;

	public SensitivityObserver(String databaseName) {
		super(databaseName);
		results = (ArrayList<String>) Config.getParameter("Observer_resultTypes");
		for (String i : results) {
			if (i.contains("sensitivity")) {
				pattern = i;
			}
		}
		mnos_ref = 0;
		fledged_ref = 0;
		laid_ref = 0;
		hatched_ref = 0;
		mnos_exp = 0;
		fledged_exp = 0;
		laid_exp = 0;
		hatched_exp = 0;
		annualShootingProbability = 0;
		pesticideSlope = 0;
		mortality = 0;
		initialPopulationSize = 0;
		year = 0;
		if (pattern.equalsIgnoreCase("sensitivity_foraging")) {
			prepareForagingDistribution();
		}
		numberOfFields = new ArrayList<Double>();
		revisitedFields = new ArrayList<Double>();
		breedingDensity = new ArrayList<Double>();
		popSize = new ArrayList<Double>();
		nestingArea = 0;
		makeTable();
	}

	private void prepareForagingDistribution() {
		foragingDistribution = new TreeMap<String, TreeMap<Integer, SortedSet<Double>>>();
		ArrayList<String> vegtypes = new ArrayList<String>();
		vegtypes.add("Cereal_sowings");
		vegtypes.add("Cereals");
		vegtypes.add("Cereal_stubble");
		vegtypes.add("Peas_and_beans");
		vegtypes.add("Winter_rape");
		vegtypes.add("Pasture");
		vegtypes.add("Clover");
		for (String veg : vegtypes) {
			TreeMap<Integer, SortedSet<Double>> map = new TreeMap<Integer, SortedSet<Double>>();
			for (int i = 1; i <= 12; i++) {
				map.put(i, new TreeSet<Double>());
			}
			foragingDistribution.put(veg, map);
		}
	}

	@Override
	public void observe(Collection<Observable> observed) {
		if (results.contains("debug")) {
			super.observe(observed);
		}
		boolean savingYear = false;
		if (pattern.equalsIgnoreCase("sensitivity_nestDestruction")) {
			savingYear = observeNestDestruction(observed);
		} else if (pattern.equalsIgnoreCase("sensitivity_shooting")) {
			savingYear = observeShooting(observed);
		} else if (pattern.equalsIgnoreCase("sensitivity_pesticide")) {
			savingYear = observePesticide(observed);
		} else if ((pattern.equalsIgnoreCase("sensitivity_foraging")
				|| results.contains("full_foraging")) && afterBurnin()) {
			observeForaging(observed);
		}
	}
	
	private boolean afterBurnin() {
		if ((World.getDate().get(GregorianCalendar.YEAR) - ((GregorianCalendar) Config.getParameter("World_startDate")).get(GregorianCalendar.YEAR) >= (Integer) Config.getParameter("Config_burnin"))) {
			return true;
		} else {
			return false;
		}
	}

	@Override
	public void shutdown() {
		finallyObserve();
		super.shutdown();
	}

	public void finallyObserve() {
		try {
			String update = "INSERT INTO " + pattern + " (starttime, ";
			update = update + insertTableParameterNames();
			if (pattern.equalsIgnoreCase("sensitivity_nestDestruction")) {
				update = update
						+ "fledged_ref, "
						+ "mnos_ref, "
						+ "fledged_exp, "
						+ "mnos_exp) "
						+ " VALUES (";
			} else if (pattern.equalsIgnoreCase("sensitivity_shooting")) {
				update = update
						+ "year, "
						+ "laid_ref, "
						+ "hatched_ref, "
						+ "laid_exp, "
						+ "hatched_exp, "
						+ "annualShooting_probability) "
						+ " VALUES (";
			} else if (pattern.equalsIgnoreCase("sensitivity_pesticide")) {
				update = update
						+ "pesticide_slope, "
						+ "pesticide_mortality) "
						+ " VALUES (";
			} else if (pattern.equalsIgnoreCase("sensitivity_foraging")) {
				for (String veg : foragingDistribution.keySet()) {
					for (Integer m : foragingDistribution.get(veg).keySet()) {
						update = update
								+ veg + Integer.toString(m) + ", ";
					}
				}
				update = update
						+ "fields_min, "
						+ "fields_max, "
						+ "fields_25, "
						+ "fields_50, "
						+ "fields_75, "
						+ "revisited_min, "
						+ "revisited_max, "
						+ "revisited_25, "
						+ "revisited_50, "
						+ "revisited_75, "
						+ "breeding_density, "
						+ "pop_size"
						+ ") VALUES (";
			}
			update = update + "'" + Config.getParameter("startTime") + "'";
			update = update + insertTableParameters();
			if (pattern.equalsIgnoreCase("sensitivity_nestDestruction")) {
				update = update
						+ ", " + prepareUpdate(fledged_ref)
						+ ", " + prepareUpdate(mnos_ref)
						+ ", " + prepareUpdate(fledged_exp)
						+ ", " + prepareUpdate(mnos_exp)
						+ ");";
			} else if (pattern.equalsIgnoreCase("sensitivity_shooting")) {
				update = update
						+ ", " + prepareUpdate(year)
						+ ", " + prepareUpdate(laid_ref)
						+ ", " + prepareUpdate(hatched_ref)
						+ ", " + prepareUpdate(laid_exp)
						+ ", " + prepareUpdate(hatched_exp)
						+ ", " + prepareUpdate(annualShootingProbability)
						+ ");";
			} else if (pattern.equalsIgnoreCase("sensitivity_pesticide")) {
				update = update
						+ ", " + prepareUpdate(pesticideSlope)
						+ ", " + prepareUpdate(mortality / initialPopulationSize)
						+ ");";
			} else if (pattern.equalsIgnoreCase("sensitivity_foraging")) {
				numberOfFields1000 = prepareResult(numberOfFields);
				revisitedFields1000 = prepareResult(revisitedFields);
				for (String veg : foragingDistribution.keySet()) {
					for (Integer m : foragingDistribution.get(veg).keySet()) {
						update = update
								+ ", " + prepareUpdate(foragingDistribution.get(veg).get(m));
					}
				}
				update = update
						+ ", " + prepareUpdate(min(numberOfFields1000))
						+ ", " + prepareUpdate(max(numberOfFields1000))
						+ ", " + prepareUpdate(percentile(numberOfFields1000, 25))
						+ ", " + prepareUpdate(percentile(numberOfFields1000, 50))
						+ ", " + prepareUpdate(percentile(numberOfFields1000, 75))
						+ ", " + prepareUpdate(min(revisitedFields1000))
						+ ", " + prepareUpdate(max(revisitedFields1000))
						+ ", " + prepareUpdate(percentile(revisitedFields1000, 25))
						+ ", " + prepareUpdate(percentile(revisitedFields1000, 50))
						+ ", " + prepareUpdate(percentile(revisitedFields1000, 75))
						+ ", " + prepareUpdate(breedingDensity)
						+ ", " + prepareUpdate(popSize)
						+ ");";
			}
			addBatch(update);
			executeBatch();
		} catch (SQLException ex) {
			Logger.getLogger(SpecificObserver.class.getName()).log(Level.SEVERE, null, ex);
		}
		if (results.contains("full_foraging")) {
			try {
				String update1 = "INSERT INTO full_foraging (starttime, ";
				update1 = update1
						+ "parameter_WoodpigeonAdult_flocking, "
						+ "parameter_WoodpigeonForaging_foraging, "
						+ "numberOfFields, "
						+ "revisitedFields) "
						+ " VALUES ("
						+ "'" + Config.getParameter("startTime") + "'"
						+ ", " + Config.getParameter("WoodpigeonAdult_flocking")
						+ ", " + "'" + Config.getParameter("WoodpigeonForaging_foraging") + "'"
						+ ", " + prepareUpdate(numberOfFields1000)
						+ ", " + prepareUpdate(revisitedFields1000)
						+ ");";

				addBatch(update1);
				executeBatch();
			} catch (SQLException ex) {
				Logger.getLogger(SpecificObserver.class.getName()).log(Level.SEVERE, null, ex);
			}
		}
	}

	private ArrayList<Double> recalculateForagingDistribution(String veg, Integer m) {
		SortedSet<Double> props = foragingDistribution.get(veg).get(m);
		ArrayList<Double> ranges = new ArrayList<Double>();
		for (Double p : props) {
			double[] r = calculateRange(p);
			if (ranges.isEmpty() || ranges.get(ranges.size() - 1) < r[0]) {
				ranges.add(r[0]);
				ranges.add(r[1]);
			} else {
				ranges.remove(ranges.size() - 1);
				ranges.add(r[1]);
			}
		}
		return ranges;
	}

	private double[] calculateRange(Double p) {
		double[] r = new double[2];
		if (p == 0) {
			r[0] = 0;
			r[1] = 0 + (Double) Config.getParameter("SensitivityObserver_rangeWidth");
		} else if (p == 1) {
			r[0] = 1 - (Double) Config.getParameter("SensitivityObserver_rangeWidth");
			r[1] = 1;
		} else {
			r[0] = p - (Double) Config.getParameter("SensitivityObserver_rangeWidth");
			r[1] = p + (Double) Config.getParameter("SensitivityObserver_rangeWidth");
		}
		return r;
	}

	public String insertTableParameterNames() {
		if (pattern.equalsIgnoreCase("sensitivity_nestDestruction")
				|| pattern.equalsIgnoreCase("sensitivity_shooting")) {
			return "parameter_WoodpigeonAdult_daysOfHungerBeforeMigration, "
					+ "parameter_WoodpigeonAnimal_imigrationProbability, "
					+ "parameter_Reproduce_readyForReproduction, "
					+ "parameter_WoodpigeonAdult_feedingRoutineLength, "
					+ "parameter_Reproduce_reproductionBuffer, "
					+ "parameter_Eggs_predationRate, "
					+ "parameter_Eggs_backgroundPredationRate, "
					+ "parameter_Nestlings_predationRate, "
					+ "parameter_LandscapeUnit_territoryGettingProbabilitySlope, "
					+ "parameter_SeasonalActivity_startOfSeason, "
					+ "parameter_SeasonalActivity_endOfSeason, "
					+ "parameter_SeasonalActivity_lateStartOfSeason, ";
		} else if (pattern.equalsIgnoreCase("sensitivity_pesticide")
				|| pattern.equalsIgnoreCase("sensitivity_foraging")) {
			return "parameter_WoodpigeonAdult_daysOfHungerBeforeMigration, "
					+ "parameter_WoodpigeonAnimal_imigrationProbability, "
					+ "parameter_Reproduce_readyForReproduction, "
					+ "parameter_WoodpigeonAdult_feedingRoutineLength, "
					+ "parameter_Reproduce_reproductionBuffer, "
					+ "parameter_WoodpigeonAdult_memoryLength, "
					+ "parameter_WoodpigeonAdult_proportionOfBetterFields, "
					+ "parameter_Config_timeFactor, "
					+ "parameter_WoodpigeonLandscapeUnit_numberOfDisturbances, "
					+ "parameter_SeasonalActivity_startOfSeason, "
					+ "parameter_SeasonalActivity_endOfSeason, "
					+ "parameter_SeasonalActivity_lateStartOfSeason, "
					+ "parameter_WoodpigeonAdult_flocking, "
					+ "parameter_WoodpigeonForaging_foraging, ";
		} else {
			return null;
		}
	}

	public String insertTableParameters() {
		if (pattern.equalsIgnoreCase("sensitivity_nestDestruction")
				|| pattern.equalsIgnoreCase("sensitivity_shooting")) {
			return ", " + Config.getParameter("WoodpigeonAdult_daysOfHungerBeforeMigration")
					+ ", " + Config.getParameter("WoodpigeonAnimal_imigrationProbability")
					+ ", " + Config.getParameter("Reproduce_readyForReproduction")
					+ ", " + Config.getParameter("WoodpigeonAdult_feedingRoutineLength")
					+ ", " + Config.getParameter("Reproduce_reproductionBuffer")
					+ ", " + Config.getParameter("Eggs_predationRate")
					+ ", " + Config.getParameter("Eggs_backgroundPredationRate")
					+ ", " + Config.getParameter("Nestlings_predationRate")
					+ ", " + Config.getParameter("LandscapeUnit_territoryGettingProbabilitySlope")
					+ ", " + Config.getParameter("SeasonalActivity_startOfSeason")
					+ ", " + Config.getParameter("SeasonalActivity_endOfSeason")
					+ ", " + Config.getParameter("SeasonalActivity_lateStartOfSeason");
		} else if (pattern.equalsIgnoreCase("sensitivity_pesticide")
				|| pattern.equalsIgnoreCase("sensitivity_foraging")) {
			return ", " + Config.getParameter("WoodpigeonAdult_daysOfHungerBeforeMigration")
					+ ", " + Config.getParameter("WoodpigeonAnimal_imigrationProbability")
					+ ", " + Config.getParameter("Reproduce_readyForReproduction")
					+ ", " + Config.getParameter("WoodpigeonAdult_feedingRoutineLength")
					+ ", " + Config.getParameter("Reproduce_reproductionBuffer")
					+ ", " + Config.getParameter("WoodpigeonAdult_memoryLength")
					+ ", " + Config.getParameter("WoodpigeonAdult_proportionOfBetterFields")
					+ ", " + Config.getParameter("Config_timeFactor")
					+ ", " + Config.getParameter("WoodpigeonLandscapeUnit_numberOfDisturbances")
					+ ", " + Config.getParameter("SeasonalActivity_startOfSeason")
					+ ", " + Config.getParameter("SeasonalActivity_endOfSeason")
					+ ", " + Config.getParameter("SeasonalActivity_lateStartOfSeason")
					+ ", " + Config.getParameter("WoodpigeonAdult_flocking")
					+ ", " + "'" + Config.getParameter("WoodpigeonForaging_foraging") + "'";
		} else {
			return null;
		}
	}

	public boolean observeNestDestruction(Collection<Observable> observed) {
		for (Calendar date : (ArrayList<GregorianCalendar>) Config.getParameter("WoodpigeonNest_destructionDates")) {
			if (World.getDate().get(Calendar.YEAR) == date.get(Calendar.YEAR) - 2) {
				int nests = World.Instance().getNumberOfAdultsAndNests()[1];
				if (nests > mnos_ref) {
					mnos_ref = nests;
				}
				for (Observable o : observed) {
					try {
						Animal a = (Animal) o;
						if (a.getDaysSinceLeavingNest() == 0) {
							fledged_ref++;
						}
					} catch (Exception e1) {
					}
				}
			}
		}
		for (Calendar date : (ArrayList<GregorianCalendar>) Config.getParameter("WoodpigeonNest_destructionDates")) {
			if (World.getDate().get(Calendar.YEAR) == date.get(Calendar.YEAR)) {
				int nests = World.Instance().getNumberOfAdultsAndNests()[1];
				if (nests > mnos_exp) {
					mnos_exp = nests;
				}
				for (Observable o : observed) {
					try {
						Animal a = (Animal) o;
						if (a.getDaysSinceLeavingNest() == 0) {
							fledged_exp++;
						}
					} catch (Exception e1) {
					}
				}
				return true;
			}
		}
		return false;
	}

	public boolean observeShooting(Collection<Observable> observed) {
		ArrayList<Integer> years = (ArrayList<Integer>) Config.getParameter("WoodpigeonAnimal_shootingYears");
		for (Integer y : years) {
			if (World.getDate().get(Calendar.YEAR) + 1 == y) {
				for (Observable o : observed) {
					try {
						Nest n = (Nest) o;
						if (n.getDaysSinceLaying() == 0) {
							laid_ref += n.getOffspringSize();
						}
						if (n.getDaysSinceHatching() == 0) {
							hatched_ref += n.getOffspringSize();
						}
					} catch (Exception e2) {
					}
				}
			} else if (World.getDate().get(Calendar.YEAR) == y) {
				for (Observable o : observed) {
					try {
						Nest n = (Nest) o;
						if (n.getDaysSinceLaying() == 0) {
							laid_exp += n.getOffspringSize();
						}
						if (n.getDaysSinceHatching() == 0) {
							hatched_exp += n.getOffspringSize();
						}
					} catch (Exception e2) {
					}
				}
				int yearIndex = years.indexOf(y);
				annualShootingProbability = 1 - Math.pow((1 - ((ArrayList<Double>) Config.getParameter("WoodpigeonAnimal_shootingProbability")).get(yearIndex)),
						((Integer) Config.getParameter("WoodpigeonAnimal_shootingEndDay")
						- (Integer) Config.getParameter("WoodpigeonAnimal_shootingStartDay") + 1));
				year = y;
				return true;
			}
		}
		return false;
	}

	public boolean observePesticide(Collection<Observable> observed) {
		if (World.getDate().get(Calendar.DAY_OF_YEAR) == (Integer) Config.getParameter("Spring_cereals_sowingDayStart")) {
			initialPopulationSize = World.Instance().getNumberOfAdultsAndNests()[0];
		}
		for (Observable o : observed) {
			try {
				Animal a = (Animal) o;
				if (((a.getSignature()).getPrimitiveField("causeOfDeath")).equalsIgnoreCase("'mortalityEffect'")) {
					mortality += 1;
				}

			} catch (Exception e1) {
			}
		}
		if (pesticideSlope == 0) {
			if (Config.getParameter("Pesticides") != null) {
				HashMap<String, Object> copy = Config.getParameterList();
				for (String pest : (ArrayList<String>) Config.getParameter("Pesticides")) {
					for (String par : copy.keySet()) {
						if (par.endsWith("_pesticide") && copy.get(par) != null) {
							if (((HashMap<String, HashMap<String, Object>>) copy.get(par)).containsKey(pest)) {
								if (Config.checkIfPesticideApplicationYear(pest, "Spring_cereals")) {
									pesticideSlope = (Double) Config.getParameter(pest + "_mortalityEffect_maxdose_slope");
									return true;
								}
							}
						}
					}
				}
			}
		}
		return false;
	}

	public void observeForaging(Collection<Observable> observed) {
		observeForagingDistribution(observed);
		observeRadiotracking(observed);
		observeBreedingDensity();
	}

	private void observeBreedingDensity() {
		if (nestingArea == 0) {
			Set<LandscapeUnit> LU = World.Instance().getLandscapeUnits();
			for (LandscapeUnit wood : LU) {
				if (wood.getStateClass().getSimpleName().equalsIgnoreCase("WoodpigeonWood")) {
					nestingArea += wood.getSize();
				}
			}
		}
		if (World.getDate().get(Calendar.DAY_OF_YEAR) == (Integer) Config.getParameter("Spring_cereals_sowingDayStart")) {
			initialPopulationSize = World.Instance().getNumberOfAdultsAndNests()[0];
		}
		int nests = World.Instance().getNumberOfAdultsAndNests()[1];
		if (nests > mnos_ref) {
			mnos_ref = nests;
		}
		if (World.getDate().get(Calendar.DAY_OF_YEAR) == 365) {
			breedingDensity.add(mnos_ref / nestingArea);
			mnos_ref = 0;
			popSize.add(initialPopulationSize);
			initialPopulationSize = 0;
		}
	}

	private void observeRadiotracking(Collection<Observable> observed) {
		if (((ArrayList<Integer>) Config.getParameter("SensitivityObserver_radiotrackingDays")).get(0) <= World.getDate().get(Calendar.DAY_OF_YEAR)
				&& ((ArrayList<Integer>) Config.getParameter("SensitivityObserver_radiotrackingDays")).get(1) >= World.getDate().get(Calendar.DAY_OF_YEAR)) {
			for (Observable o : observed) {
				try {
					WoodpigeonAnimal a = (WoodpigeonAnimal) o;
					if (a.getAlive()
							&& a.getDaysSinceLaying() > (Integer) Config.getParameter("WoodpigeonAdult_startOfAdulthood")
							&& a.getSignature().getPrimitiveField("migration").equalsIgnoreCase("false")) {
						Signature s = a.getSignature();
						HashSet<Signature> subSigs = s.getInternalObjects();
						Iterator<Signature> it = subSigs.iterator();
						while (it.hasNext()) {
							Signature stateSig = it.next();
							if (stateSig.getPrimitiveField("Signatory").contains("Adult")) {
								if (!stateSig.getPrimitiveField("revisitedFields").equalsIgnoreCase("null")) {
									revisitedFields.add(Double.valueOf(stateSig.getPrimitiveField("revisitedFields")));
								}
								numberOfFields.add(Double.valueOf(stateSig.getPrimitiveField("numberOfFields")));
							}
						}
					}
				} catch (Exception e1) {
				}
			}
		}
	}

	private void observeForagingDistribution(Collection<Observable> observed) {
		for (Calendar date : (ArrayList<GregorianCalendar>) Config.getParameter("SensitivityObserver_carltonDates")) {
			if (World.calculateDateDifference(date) == 0) {
				int month = date.get(Calendar.MONTH) + 1;
				int timeInterval = (int) Math.floor(Config.getGenerator().nextDouble() * Config.getDayLength(date));
				HashMap<String, Double> vegTypes = new HashMap<String, Double>();
				double pop_observed = 0;
				vegTypes.put("Cereal_sowings", 0.0);
				vegTypes.put("Cereals", 0.0);
				vegTypes.put("Cereal_stubble", 0.0);
				vegTypes.put("Peas_and_beans", 0.0);
				vegTypes.put("Winter_rape", 0.0);
				vegTypes.put("Pasture", 0.0);
				vegTypes.put("Clover", 0.0);
				for (Observable o : observed) {
					try {
						WoodpigeonLandscapeUnit lu = (WoodpigeonLandscapeUnit) o;
						int flock = lu.getFlockSize(timeInterval);
						pop_observed += flock;
						HashSet<Signature> sign = lu.getSignature().getInternalObjects();
						for (Signature s : sign) {
							if (s.getPrimitiveFields().containsKey("carltonVeg")) {
								String veg = (s.getPrimitiveField("carltonVeg")).substring(1, s.getPrimitiveField("carltonVeg").length() - 1);
								if (vegTypes.containsKey(veg)) {
									vegTypes.put(veg, vegTypes.get(veg) + flock);
								}
							}
						}
					} catch (Exception e1) {
					}
				}

				for (String vege : vegTypes.keySet()) {
					double proportion;
					if (pop_observed > 0) {
						proportion = vegTypes.get(vege) / pop_observed;
					} else {
						proportion = 0.0;
					}
					foragingDistribution.get(vege).get(month).add(proportion);
				}
			}
		}
	}

	public void makeTable() {
		String statement = "CREATE TABLE IF NOT EXISTS " + pattern + " ("
				+ "starttime text, ";
		statement = statement + makeTableParameters();
		if (pattern.equalsIgnoreCase("sensitivity_nestDestruction")) {
			statement = statement
					+ "fledged_ref double precision, "
					+ "mnos_ref double precision, "
					+ "fledged_exp double precision, "
					+ "mnos_exp double precision, "
					+ "PRIMARY KEY (starttime));";
		} else if (pattern.equalsIgnoreCase("sensitivity_shooting")) {
			statement = statement
					+ "year integer, "
					+ "laid_ref double precision, "
					+ "hatched_ref double precision, "
					+ "laid_exp double precision, "
					+ "hatched_exp double precision, "
					+ "annualShooting_probability double precision, "
					+ "PRIMARY KEY (starttime));";
		} else if (pattern.equalsIgnoreCase("sensitivity_pesticide")) {
			statement = statement
					+ "pesticide_slope double precision, "
					+ "pesticide_mortality double precision, "
					+ "PRIMARY KEY (starttime));";
		} else if (pattern.equalsIgnoreCase("sensitivity_foraging")) {
			for (String veg : foragingDistribution.keySet()) {
				for (Integer m : foragingDistribution.get(veg).keySet()) {
					statement = statement
							+ veg + Integer.toString(m) + " double precision[], ";
				}
			}
			statement = statement
					+ "fields_min double precision, "
					+ "fields_max double precision, "
					+ "fields_25 double precision, "
					+ "fields_50 double precision, "
					+ "fields_75 double precision, "
					+ "revisited_min double precision, "
					+ "revisited_max double precision, "
					+ "revisited_25 double precision, "
					+ "revisited_50 double precision, "
					+ "revisited_75 double precision, "
					+ "breeding_density double precision[], "
					+ "pop_size double precision[], "
					+ "PRIMARY KEY (starttime));";
		}
		try {
			executeStatement(statement);
		} catch (SQLException e) {
			e.printStackTrace();
			e.getNextException().printStackTrace();
		}
		if (results.contains("full_foraging")) {
			String statement1 = "CREATE TABLE IF NOT EXISTS full_foraging ("
					+ "starttime text, ";
			statement1 = statement1
					+ "parameter_WoodpigeonAdult_flocking boolean, "
					+ "parameter_WoodpigeonForaging_foraging text, "
					+ "numberOfFields double precision[], "
					+ "revisitedFields double precision[], "
					+ "PRIMARY KEY (starttime));";
			try {
				executeStatement(statement1);
			} catch (SQLException e) {
				e.printStackTrace();
				e.getNextException().printStackTrace();
			}
		}
	}

	public String makeTableParameters() {
		if (pattern.equalsIgnoreCase("sensitivity_nestDestruction")
				|| pattern.equalsIgnoreCase("sensitivity_shooting")) {
			return "parameter_WoodpigeonAdult_daysOfHungerBeforeMigration integer, "
					+ "parameter_WoodpigeonAnimal_imigrationProbability double precision, "
					+ "parameter_Reproduce_readyForReproduction double precision, "
					+ "parameter_WoodpigeonAdult_feedingRoutineLength integer, "
					+ "parameter_Reproduce_reproductionBuffer double precision, "
					+ "parameter_Eggs_predationRate double precision, "
					+ "parameter_Eggs_backgroundPredationRate double precision, "
					+ "parameter_Nestlings_predationRate double precision, "
					+ "parameter_LandscapeUnit_territoryGettingProbabilitySlope double precision, "
					+ "parameter_SeasonalActivity_startOfSeason integer, "
					+ "parameter_SeasonalActivity_endOfSeason integer, "
					+ "parameter_SeasonalActivity_lateStartOfSeason integer, ";
		} else if (pattern.equalsIgnoreCase("sensitivity_pesticide")
				|| pattern.equalsIgnoreCase("sensitivity_foraging")) {
			return "parameter_WoodpigeonAdult_daysOfHungerBeforeMigration integer, "
					+ "parameter_WoodpigeonAnimal_imigrationProbability double precision, "
					+ "parameter_Reproduce_readyForReproduction double precision, "
					+ "parameter_WoodpigeonAdult_feedingRoutineLength integer, "
					+ "parameter_Reproduce_reproductionBuffer double precision, "
					+ "parameter_WoodpigeonAdult_memoryLength integer, "
					+ "parameter_WoodpigeonAdult_proportionOfBetterFields double precision, "
					+ "parameter_Config_timeFactor integer, "
					+ "parameter_WoodpigeonLandscapeUnit_numberOfDisturbances integer, "
					+ "parameter_SeasonalActivity_startOfSeason integer, "
					+ "parameter_SeasonalActivity_endOfSeason integer, "
					+ "parameter_SeasonalActivity_lateStartOfSeason integer, "
					+ "parameter_WoodpigeonAdult_flocking boolean, "
					+ "parameter_WoodpigeonForaging_foraging text, ";
		} else {
			return null;
		}
	}
}