//Copyright 2013 Katarzyna Kułakowska, Tomasz Kułakowski.
//
//This file is part of SkyPig.
//
//SkyPig is free software; you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation; either version 2 of the License, or
//(at your option) any later version.
//
//SkyPig is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with SkyPig; if not, write to the Free Software
//Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

package model.woodpigeon;

import java.util.TreeMap;
import java.util.logging.Level;
import model.core.*;
import model.woodpigeon.WoodpigeonAnimal.WoodpigeonAdult;

/**
 *
 * @author kasia
 */
public class WoodpigeonForaging {

	/*
	 * Fields:
	 */
	/**
	 *
	 */
	private static WoodpigeonForagingStrategy foraging = null;

	/*
	 * Methods:
	 */
	/**
	 * 
	 * @param requirements
	 * @param me
	 * @return
	 * @verification Warning if the foraging strategy passed in configuration 
	 * file is not implemented.
	 */
	public static LandscapeUnit chooseBestLU(double requirements, WoodpigeonAdult me) {
		if (foraging == null) {
			if (((String) Config.getParameter("WoodpigeonForaging_foraging")).equalsIgnoreCase("memoryForaging")) {
				foraging = new MemoryForaging();
			} else if (((String) Config.getParameter("WoodpigeonForaging_foraging")).equalsIgnoreCase("optimalForaging")) {
				foraging = new OptimalForaging();
			} else if (((String) Config.getParameter("WoodpigeonForaging_foraging")).equalsIgnoreCase("randomForaging")) {
				foraging = new RandomForaging();
			} else {
// *****VERIFICATION MODE START*****//
				if (World.verificationMode) {
					// ***TEST 1
					try {
						throw new SkyPigException(
								"Improper foreging strategy: " + ((String) Config.getParameter("WoodpigeonForaging_foraging")));
					} catch (SkyPigException ex) {
						World.log(Level.WARNING, Config.getLogInfo(), ex);
					}
				}
			}
// *****VERIFICATION MODE END*****//				
		}
		return foraging.chooseBestLU(requirements, me);
	}
	
	/**
	 * 
	 * @param energyAssimilationRate
	 * @param requirements
	 * @param me
	 * @return
	 */
	public static boolean achieveOptimalEnergyAssimilationRate(double energyAssimilationRate, double requirements, WoodpigeonAdult me) {
		return foraging.achieveOptimalEnergyAssimilationRate(energyAssimilationRate, requirements, me);
	}
	

	/**
	 * 
	 */
	public interface WoodpigeonForagingStrategy {

		/**
		 * 
		 * @param requirements
		 * @param me
		 * @return
		 */
		public LandscapeUnit chooseBestLU(double requirements, WoodpigeonAdult me);
		/**
		 * 
		 * @param energyAssimilationRate
		 * @param requirements
		 * @param me
		 * @return
		 */
		public boolean achieveOptimalEnergyAssimilationRate(double energyAssimilationRate, double requirements, WoodpigeonAdult me);
	}

	/**
	 * 
	 */
	public static class MemoryForaging implements WoodpigeonForagingStrategy {

		/**
		 * 
		 * @param requirements
		 * @param me
		 * @return
		 */
		@Override
		public LandscapeUnit chooseBestLU(double requirements, WoodpigeonAdult me) {
			LandscapeUnit x = null;
			double betterFields = 0;
			double worseFields = 0;
			double proportionOfBetterFields = 0;
			double total = 0;
			TreeMap<Double, LandscapeUnit> sums = new TreeMap<Double, LandscapeUnit>();
			if (!me.getMemory().isEmpty()) {
				for (double speed : me.getMemory().keySet()) {
					for (WoodpigeonAnimal.Memorable m : me.getMemory().get(speed)) {
						if (speed >= requirements / (double) me.timeLeftForForagingToday) {
							betterFields++;
						} else {
							worseFields++;
						}
						total += speed;
						sums.put(total, m.getLu());
					}
				}
				proportionOfBetterFields = betterFields / (betterFields + worseFields);
			}
			if (Config.getGenerator().nextDouble() < proportionOfBetterFields) {
				x = sums.get(sums.ceilingKey(Config.getGenerator().nextDouble() * total));
			} else {
				x = World.getRandomLandscapeUnit();
			}
			return x;
		}
		/**
		 * 
		 * @param energyAssimilationRate
		 * @param requirements
		 * @param me
		 * @return
		 */
		@Override
		public boolean achieveOptimalEnergyAssimilationRate(double energyAssimilationRate, double requirements, WoodpigeonAdult me) {
			if (energyAssimilationRate >= requirements / me.timeLeftForForagingToday) {
				return true;
			} else {
				double betterFields = 0;
				double worseFields = 0;
				double proportionOfBetterFields = 0;
				double bestSpeed = 0;
				if (!me.getMemory().isEmpty()) {
					for (double speed : me.getMemory().keySet()) {
						for (WoodpigeonAnimal.Memorable m : me.getMemory().get(speed)) {
							if (speed > energyAssimilationRate) {
								betterFields++;
							} else {
								worseFields++;
							}
							if (speed > bestSpeed) {
								bestSpeed = speed;
							}
						}
					}
					proportionOfBetterFields = betterFields / (betterFields + worseFields);
				}
				if (bestSpeed > requirements / me.timeLeftForForagingToday
						|| proportionOfBetterFields > (Double) Config.getParameter("WoodpigeonAdult_proportionOfBetterFields")) {
					return false;
				} else {
					return true;
				}
			}
		}
	}

	/**
	 * 
	 */
	public static class OptimalForaging implements WoodpigeonForagingStrategy {

		/**
		 * 
		 * @param requirements
		 * @param me
		 * @return
		 */
		@Override
		public LandscapeUnit chooseBestLU(double requirements, WoodpigeonAdult me) {
			LandscapeUnit x = null;
			double energy = 0.0;
			for (LandscapeUnit lu : World.Instance().getLandscapeUnits()) {
				Resource maxResource = me.chooseBestResource(lu);
				int cropFillingTime = me.calculateCropFillingTime(maxResource, lu);
				double energyAssimilationRate = me.getEnergyIntakeRate(maxResource, lu, cropFillingTime);
				if (energyAssimilationRate > energy) {
					x = lu;
					energy = energyAssimilationRate;
				}
			}
			return x;
		}
		
		/**
		 * 
		 * @param energyAssimilationRate
		 * @param requirements
		 * @param me
		 * @return
		 */
		@Override
		public boolean achieveOptimalEnergyAssimilationRate(double energyAssimilationRate, double requirements, WoodpigeonAdult me) {
			return true;
		}
	}

	/**
	 * 
	 */
	public static class RandomForaging implements WoodpigeonForagingStrategy {

		/**
		 * 
		 * @param requirements
		 * @param me
		 * @return
		 */
		@Override
		public LandscapeUnit chooseBestLU(double requirements, WoodpigeonAdult me) {
			return World.getRandomLandscapeUnit();
		}
		
		/**
		 * 
		 * @param energyAssimilationRate
		 * @param requirements
		 * @param me
		 * @return
		 */
		@Override
		public boolean achieveOptimalEnergyAssimilationRate(double energyAssimilationRate, double requirements, WoodpigeonAdult me) {
			if (energyAssimilationRate >= requirements / me.timeLeftForForagingToday) {
				return true;
			} else {
				return false;
			}
		}
	}
}
