//Copyright 2013 Katarzyna Kułakowska, Tomasz Kułakowski.
//
//This file is part of SkyPig.
//
//SkyPig is free software; you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation; either version 2 of the License, or
//(at your option) any later version.
//
//SkyPig is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with SkyPig; if not, write to the Free Software
//Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
package model.woodpigeon;

import java.util.*;
import java.util.logging.Level;
import model.core.*;

/**
 *
 * @author kasia
 */
public class WoodpigeonLandscapeUnit extends LandscapeUnit {

	/**
	 *
	 */
	public abstract class WoodpigeonVegetation extends Vegetation {

		private Double oilseedGrowthSpeed;
		/**
		 *
		 */
		public Double lengthOfFirstSummeraDay;
		/**
		 *
		 */
		public Double lengthOfCloverGrowthStartDay;
		private boolean carltonFallow;

		/**
		 *
		 * @param parameters
		 */
		public WoodpigeonVegetation(HashMap<String, Object> parameters) {
			super(parameters);
			oilseedGrowthSpeed = null;
			lengthOfCloverGrowthStartDay = null;
			lengthOfFirstSummeraDay = null;
			carltonFallow = false;
		}

		/**
		 *
		 * @param listOfResources
		 * @return
		 */
		public HashMap<String, Double> provideListOfResources(String[] listOfResources) {
			HashMap<String, Double> list = new HashMap<String, Double>();
			for (String res : listOfResources) {
				list.put(res, 0.0);
			}
			return list;
		}

		@Override
		public void reset() {
			resetTimetable();
			resetDisturbanceTimes();
			super.reset();
		}

		/**
		 *
		 * @param resourceName
		 */
		public void harvestLeaves(String resourceName) {
			for (Resource r : getResources()) {
				if (r.getName().equalsIgnoreCase(resourceName)) {
					changeResourceAmount(resourceName, 0, false);
					changeResourceEnergyContent(resourceName, 0);
				}
			}
		}

		/**
		 *
		 * @param resourceName
		 */
		public void harvestSeeds(String resourceName) {
			for (Resource r : getResources()) {
				if (r.getName().equalsIgnoreCase(resourceName)) {
					double newAmount = (Double) Config.getParameter(r.getName() + "_afterHarvestDensity")
							* getSize();
					changeResourceAmount(resourceName, newAmount, false);
				}
			}
		}

		/**
		 *
		 * @param resourceName
		 */
		public void sowSeeds(String resourceName) {
			for (Resource r : getResources()) {
				if (r.getName().equalsIgnoreCase(resourceName)) {
					double newAmount = (Double) Config.getParameter(r.getName() + "_sowingDensity")
							* (Double) Config.getParameter(r.getName() + "_sowingInefficiency") * getSize();
					double newEnergyContent = (Double) Config.getParameter(r.getName() + "_metabolizableEnergy");
					changeResourceAmount(resourceName, newAmount, true);
					changeResourceEnergyContent(resourceName, newEnergyContent);
					changeResourceDensity(resourceName, newAmount * (Double) Config.getParameter(r.getName() + "_afterSowingHighDensity") / getSize());
					if (Config.getParameter("Pesticides") != null) {
						if (((HashMap<String, HashMap<String, Object>>) Config.getParameter(name + "_pesticide")) != null) {
							for (String pesticide : (ArrayList<String>) Config.getParameter("Pesticides")) {
								if (((HashMap<String, HashMap<String, Object>>) Config.getParameter(name + "_pesticide")).containsKey(pesticide)) {
									if (((String) ((HashMap<String, HashMap<String, Object>>) Config.getParameter(name + "_pesticide")).get(pesticide).get("mode")).equalsIgnoreCase("seedtreatment")) {
										if (Config.checkIfPesticideApplicationYear(pesticide, name) && Config.getGenerator().nextDouble() < ((Double) ((HashMap<String, HashMap<String, Object>>) Config.getParameter(name + "_pesticide")).get(pesticide).get("probability"))) {											double pesticideAmount = (Double) (((HashMap<String, HashMap<String, Object>>) Config.getParameter(name + "_pesticide")).get(pesticide)).get("density")
													* newAmount * (Double) Config.getParameter(pesticide + "_loadFactor") / 1000.0;
											r.addPesticide(pesticide, pesticideAmount);
										}
									}
								}
							}
						}
					}
				}
			}
			super.sow();
		}

		/**
		 *
		 * @param resourceName @verification TEST 1 Warning if the agricultural
		 * time is wrongly specified.
		 */
		public void manageEnergyForSeeds(String resourceName) {
			for (Resource r : getResources()) {
				if (r.getName().equalsIgnoreCase(resourceName)) {
					if (agriculturalTime.equalsIgnoreCase("afterSowing")) {
						if (calculateTimeDifference(harvestingDay, World.getDate().get(Calendar.DAY_OF_YEAR))
								== (Integer) Config.getParameter(r.getName() + "_timeToRipen")) {
							changeResourceAmount(resourceName, 0, false);
							double newAmount = (Double) Config.getParameter(r.getName() + "_densityUnharvested") * getSize();
							changeResourceEnergyContent(resourceName, 0);
							changeResourceAmount(resourceName, newAmount, false);
						} else if (calculateTimeDifference(harvestingDay, World.getDate().get(Calendar.DAY_OF_YEAR))
								< (Integer) Config.getParameter(r.getName() + "_timeToRipen")) {
							double newEnergyContent = r.energyContent + ((Double) Config.getParameter(r.getName() + "_metabolizableEnergy")
									/ ((Integer) Config.getParameter(r.getName() + "_timeToRipen")).doubleValue());
							changeResourceEnergyContent(resourceName, newEnergyContent);
							changeResourceAmount(resourceName, r.amount, false);
						} else {
							double newAmount = r.amount * Math.pow(0.5,
									(1.0 / (Integer) Config.getParameter(r.getName() + "_halfLifeAfterSowing")));
							changeResourceAmount(resourceName, newAmount, true);
							changeResourceDensity(resourceName, newAmount * (Double) Config.getParameter(r.getName() + "_afterSowingHighDensity") / getSize());
						}
					} else if (agriculturalTime.equalsIgnoreCase("afterHarvest")) {
						double newAmount = r.amount * Math.pow((0.5),
								(1.0 / (Integer) Config.getParameter(r.getName() + "_halfLifeAfterHarvest")));
						changeResourceAmount(resourceName, newAmount, false);
					} else {
// *****VERIFICATION MODE START*****//
						if (World.verificationMode) {
							// ***TEST 1
							try {
								throw new SkyPigException(
										"Improper agricultural time: " + agriculturalTime);
							} catch (SkyPigException ex) {
								World.log(Level.WARNING, Config.getLogInfo(), ex);
							}
						}
// *****VERIFICATION MODE END*****//
					}
				}
			}
		}

		/**
		 *
		 * @param resourceName
		 */
		public void manageEnergyForOilseedRape(String resourceName) {
			for (Resource r : getResources()) {
				if (r.getName().equalsIgnoreCase(resourceName)) {
					if (agriculturalTime.equalsIgnoreCase("afterSowing")) {
						changeResourceAmount(resourceName, r.amount + calculateGrowthSpeed(r), true);
					} else if (agriculturalTime.equalsIgnoreCase("afterHarvest")) {
						changeResourceAmount(resourceName, 0, false);
					}
				}
			}
		}

		private double calculateGrowthSpeed(Resource r) {
			if (calculateTimeDifference(World.getDate().get(Calendar.DAY_OF_YEAR), sowingDay)
					< (Integer) Config.getParameter(r.getName() + "_timeToGermination")) {
				return 0.0;
			} else {
				if (World.getDate().get(Calendar.DAY_OF_YEAR)
						< (Integer) Config.getParameter(r.getName() + "_startOfGrowthPhase")) {
					return (Double) Config.getParameter(r.getName() + "_lateWinterGrowthRate") * getSize();
				} else if (World.getDate().get(Calendar.DAY_OF_YEAR)
						< (Integer) Config.getParameter(r.getName() + "_maxDensityDay")) {
					if (oilseedGrowthSpeed == null) {
						oilseedGrowthSpeed = ((Double) Config.getParameter(r.getName() + "_maxDensity") * getSize() - r.amount)
								/ Double.valueOf((Integer) Config.getParameter(r.getName() + "_maxDensityDay")
								- World.getDate().get(Calendar.DAY_OF_YEAR));
					}
					return oilseedGrowthSpeed;
				} else if (World.getDate().get(Calendar.DAY_OF_YEAR) <= harvestingDay) {
					return -(Double) Config.getParameter(r.getName() + "_maxDensity") * getSize()
							/ Double.valueOf(harvestingDay
							- (Integer) Config.getParameter(r.getName() + "_maxDensityDay"));
				} else if (World.getDate().get(Calendar.DAY_OF_YEAR)
						< (Integer) Config.getParameter(r.getName() + "_stopOfGrowthPhase")) {
					return (Double) Config.getParameter(r.getName() + "_pauseDensity") * getSize()
							/ Double.valueOf((Integer) Config.getParameter(r.getName() + "_stopOfGrowthPhase")
							- (sowingDay + (Integer) Config.getParameter(r.getName() + "_timeToGermination")));
				} else {
					return (Double) Config.getParameter(r.getName() + "_earlyWinterGrowthRate") * getSize();
				}
			}
		}

		/**
		 *
		 * @param resourceName
		 */
		public void manageEnergyForClover(String resourceName) {
			for (Resource r : getResources()) {
				if (r.getName().equalsIgnoreCase(resourceName)) {
					if ((r.amount / getSize()) < (Double) Config.getParameter(r.getName() + "_maxDensity")) {
						if (Config.getRealDayLength(World.getDate()) > lengthOfCloverGrowthStartDay) {
							double newAmount = r.amount + ((Config.getRealDayLength(World.getDate())
									- lengthOfCloverGrowthStartDay)
									/ (lengthOfFirstSummeraDay - lengthOfCloverGrowthStartDay))
									* (Double) Config.getParameter(r.getName() + "_maxDailyGrowth");
							changeResourceAmount(resourceName, newAmount, true);
						} else {
							//	double newAmount = 0.99 * r.amount;
							//	changeResourceAmount(resourceName, newAmount);
						}
					} else {
						changeResourceAmount(resourceName, r.amount
								* (Double) Config.getParameter(r.getName() + "_mowingRatio"), true);
					}
				}
			}
		}

		@Override
		public Signature getSignature() {
			Signature newSignature = super.getSignature();
			String carltonVeg = name;
			if (name.equalsIgnoreCase("Winter_cereals") || name.equalsIgnoreCase("Spring_cereals")) {
				if (agriculturalTime.equalsIgnoreCase("afterSowing")) {
					if (calculateTimeDifference(World.getDate().get(Calendar.DAY_OF_YEAR), sowingDay) < 15) {
						carltonVeg = "Cereal_sowings";
					} else {
						carltonVeg = "Cereals";
					}
				} else if (agriculturalTime.equalsIgnoreCase("afterHarvest")) {
					if (World.calculateDateDifference(new GregorianCalendar(World.getDate().get(Calendar.YEAR), 11, 25)) >= 0
							|| World.getDate().get(Calendar.DAY_OF_YEAR) < (Integer)Config.getParameter("Vegetation_halfYear")) {
						carltonFallow = true;
					}
					if (carltonFallow) {
						carltonVeg = "Fallow";
					} else {
						carltonVeg = "Cereal_stubble";
					}
				}
			}
			newSignature.addPrimitiveField("carltonVeg", "text", "'" + carltonVeg + "'");
			return newSignature;
		}
	}

	/**
	 *
	 */
	public class WoodpigeonWinterCereals extends WoodpigeonVegetation {

		/**
		 *
		 * @param parameters
		 */
		public WoodpigeonWinterCereals(HashMap<String, Object> parameters) {
			super(parameters);
		}

		public HashMap<String, Double> getListOfResources() {
			String[] listOfResources = {"WinterWheatGrain"};
			return provideListOfResources(listOfResources);
		}

		@Override
		public void sow() {
			sowSeeds("WinterWheatGrain");
			super.sow();
		}

		@Override
		public void harvest() {
			harvestSeeds("WinterWheatGrain");
			super.harvest();
		}

		/**
		 * Changes the amount and energy content of wheat resources (wheat
		 * grain).
		 *
		 * After sowing the amount of grain on the fields diminishes
		 * independently from woodpigeons at a rate determined by half life
		 * time. When it reaches density at which pigeons do not feed on it, the
		 * amount is changed to 0. Energy content is not changing. Before
		 * ripening it is assumed grains appear while flowering in the amount
		 * (weight) fount there at harvest. But the energy content of the grain
		 * changes linearly from 0 to max energy content found in harvested
		 * grain.
		 *
		 */
		@Override
		public void manageEnergy() {
			super.manageEnergy();
			manageEnergyForSeeds("WinterWheatGrain");
		}
	}

	/**
	 *
	 */
	public class WoodpigeonSpringCereals extends WoodpigeonVegetation {

		/**
		 *
		 * @param parameters
		 */
		public WoodpigeonSpringCereals(HashMap<String, Object> parameters) {
			super(parameters);
		}

		public HashMap<String, Double> getListOfResources() {
			String[] listOfResources = {"SpringWheatGrain"};
			return provideListOfResources(listOfResources);
		}

		@Override
		public void sow() {
			sowSeeds("SpringWheatGrain");
			super.sow();
		}

		@Override
		public void harvest() {
			harvestSeeds("SpringWheatGrain");
			super.harvest();
		}

		/**
		 * Changes the amount and energy content of wheat resources (wheat
		 * grain).
		 *
		 * After sowing the amount of grain on the fields diminishes
		 * independently from woodpigeons at a rate determined by half life
		 * time. When it reaches density at which pigeons do not feed on it, the
		 * amount is changed to 0. Energy content is not changing. Before
		 * ripening it is assumed grains appear while flowering in the amount
		 * (weight) fount there at harvest. But the energy content of the grain
		 * changes linearly from 0 to max energy content found in harvested
		 * grain.
		 *
		 */
		@Override
		public void manageEnergy() {
			super.manageEnergy();
			manageEnergyForSeeds("SpringWheatGrain");

		}
	}

	/**
	 *
	 */
	public class WoodpigeonSpringOilseedRape extends WoodpigeonVegetation {

		/**
		 *
		 * @param parameters
		 */
		public WoodpigeonSpringOilseedRape(HashMap<String, Object> parameters) {
			super(parameters);
		}

		public HashMap<String, Double> getListOfResources() {
			String[] listOfResources = {"SpringRapeLeaves"};
			return provideListOfResources(listOfResources);
		}

		@Override
		public void manageEnergy() {
			super.manageEnergy();
			manageEnergyForOilseedRape("SpringRapeLeaves");
		}

		@Override
		public void harvest() {
			harvestLeaves("SpringRapeLeaves");
			super.harvest();
		}
	}

	/**
	 *
	 */
	public class WoodpigeonWinterOilseedRape extends WoodpigeonVegetation {

		/**
		 *
		 * @param parameters
		 */
		public WoodpigeonWinterOilseedRape(HashMap<String, Object> parameters) {
			super(parameters);
		}

		public HashMap<String, Double> getListOfResources() {
			String[] listOfResources = {"WinterRapeLeaves"};
			return provideListOfResources(listOfResources);
		}

		@Override
		public void manageEnergy() {
			super.manageEnergy();
			manageEnergyForOilseedRape("WinterRapeLeaves");
		}

		@Override
		public void harvest() {
			harvestLeaves("WinterRapeLeaves");
			super.harvest();
		}
	}

	/**
	 *
	 */
	public class WoodpigeonClover extends WoodpigeonVegetation {

		/**
		 *
		 * @param parameters
		 */
		public WoodpigeonClover(HashMap<String, Object> parameters) {
			super(parameters);
			Calendar startOfGrowthPhase = new GregorianCalendar(World.getDate().get(Calendar.YEAR), 0, 1);
			startOfGrowthPhase.add(GregorianCalendar.DAY_OF_YEAR, (Integer) Config.getParameter("CloverLeaves_startOfGrowthPhase"));
			Calendar firstSummerDay = new GregorianCalendar(2007, 5, 21);
			lengthOfCloverGrowthStartDay = Config.getRealDayLength(startOfGrowthPhase);
			lengthOfFirstSummeraDay = Config.getRealDayLength(firstSummerDay);
		}

		public HashMap<String, Double> getListOfResources() {
			String[] listOfResources = {"CloverLeaves"};
			return provideListOfResources(listOfResources);
		}

		/**
		 * Changes the clover resources on the daily basis.
		 *
		 * The growth of clover leaves depends on photoperiod. Between 21 March
		 * and 23 September the growth of clover equals: maxCloverGrowth *
		 * (todaysDayLength-dayLength on 21March)/max DayLength. Between 23
		 * September and 21 March clover is not growing. If the maximum clover
		 * density is reached, clover stops growing.
		 *
		 */
		@Override
		public void manageEnergy() {
			super.manageEnergy();
			manageEnergyForClover("CloverLeaves");
		}

		@Override
		public void sow() {
			Set<Resource> resources = getResources();
			for (Resource r : resources) {
				if (r.getName().equalsIgnoreCase("CloverLeaves")) {
					changeResourceAmount("CloverLeaves", (Double) Config.getParameter("CloverLeaves_pauseDensity") * getSize(), true);
				}
			}
			super.sow();
		}
	}

	/**
	 *
	 */
	public class WoodpigeonWood extends WoodpigeonVegetation {

		/**
		 *
		 * @param parameters
		 */
		public WoodpigeonWood(HashMap<String, Object> parameters) {
			super(parameters);
		}

		@Override
		public HashMap<String, Double> getListOfResources() {
			return null;
		}

		@Override
		public Message readMessage(Message m) {
			if (m.topic.equalsIgnoreCase("dead_agent") || m.topic.equalsIgnoreCase("remove_agent")) {
				territories.remove(m.senderId);
				return super.readMessage(m);
			} else {
				return super.readMessage(m);
			}
		}
	}

	/**
	 *
	 */
	public class WoodpigeonPasture extends WoodpigeonVegetation {

		/**
		 *
		 * @param parameters
		 */
		public WoodpigeonPasture(HashMap<String, Object> parameters) {
			super(parameters);
			Calendar startOfGrowthPhase = new GregorianCalendar(World.getDate().get(Calendar.YEAR), 0, 1);
			startOfGrowthPhase.add(GregorianCalendar.DAY_OF_YEAR, (Integer) Config.getParameter("PastureClover_startOfGrowthPhase"));
			Calendar firstSummerDay = new GregorianCalendar(2007, 5, 21);
			lengthOfCloverGrowthStartDay = Config.getRealDayLength(startOfGrowthPhase);
			lengthOfFirstSummeraDay = Config.getRealDayLength(firstSummerDay);
		}

		public HashMap<String, Double> getListOfResources() {
			String[] listOfResources = {"PastureClover"};
			return provideListOfResources(listOfResources);
		}

		@Override
		public void manageEnergy() {
			super.manageEnergy();
			manageEnergyForClover("PastureClover");
		}

		@Override
		public void sow() {
			Set<Resource> resources = getResources();
			for (Resource r : resources) {
				if (r.getName().equalsIgnoreCase("PastureClover")) {
					changeResourceAmount("PastureClover", (Double) Config.getParameter("PastureClover_pauseDensity") * getSize(), true);
				}
			}
			super.sow();
		}
	}

	/**
	 *
	 */
	public class WoodpigeonBrassicaceae extends WoodpigeonVegetation {

		/**
		 *
		 * @param parameters
		 */
		public WoodpigeonBrassicaceae(HashMap<String, Object> parameters) {
			super(parameters);
		}

		public HashMap<String, Double> getListOfResources() {
			return null;
		}

		@Override
		public void manageEnergy() {
			super.manageEnergy();
		}
	}

	/**
	 *
	 */
	public class WoodpigeonPeasAndBeans extends WoodpigeonVegetation {

		/**
		 *
		 * @param parameters
		 */
		public WoodpigeonPeasAndBeans(HashMap<String, Object> parameters) {
			super(parameters);
		}

		public HashMap<String, Double> getListOfResources() {
			String[] listOfResources = {"BeansSeeds"};
			return provideListOfResources(listOfResources);
		}

		@Override
		public void sow() {
			sowSeeds("BeansSeeds");
			super.sow();
		}

		@Override
		public void harvest() {
			harvestSeeds("BeansSeeds");
			super.harvest();
		}

		/**
		 * Changes the amount and energy content of wheat resources (wheat
		 * grain).
		 *
		 * After sowing the amount of grain on the fields diminishes
		 * independently from woodpigeons at a rate determined by half life
		 * time. When it reaches density at which pigeons do not feed on it, the
		 * amount is changed to 0. Energy content is not changing. Before
		 * ripening it is assumed grains appear while flowering in the amount
		 * (weight) fount there at harvest. But the energy content of the grain
		 * changes linearly from 0 to max energy content found in harvested
		 * grain.
		 *
		 */
		@Override
		public void manageEnergy() {
			super.manageEnergy();
			manageEnergyForSeeds("BeansSeeds");
		}
	}

	/**
	 *
	 */
	public class WoodpigeonFallow extends WoodpigeonVegetation {

		/**
		 *
		 * @param parameters
		 */
		public WoodpigeonFallow(HashMap<String, Object> parameters) {
			super(parameters);
			setIntermediateFallowPloughingDay();
		}

		@Override
		public AgentState changeState() {
			AgentState next = getNextStateForFallow();
			if (next == null) {
				next = super.changeState();
			}
			return next;
		}

		public HashMap<String, Double> getListOfResources() {
			return null;
		}
	}

	/**
	 *
	 */
	public class WoodpigeonOther extends WoodpigeonVegetation {

		/**
		 *
		 * @param parameters
		 */
		public WoodpigeonOther(HashMap<String, Object> parameters) {
			super(parameters);
		}

		public HashMap<String, Double> getListOfResources() {
			return null;
		}
	}
	/*
	 * Fields
	 */
	private int[] timetable;
	/**
	 * The number of TimetableSlots available for today;
	 */
	private int photoperiod;
	/**
	 * Set of Id of male birds having territories on this LandscapeUnit.
	 */
	private HashSet<Long> territories;
	private TreeSet<Integer> disturbanceTimes;
	private double meanNumberOfTerritories;

	/**
	 *
	 * @param parameters
	 */
	public WoodpigeonLandscapeUnit(HashMap<String, Object> parameters) {
		super(parameters);
		timetable = new int[Config.getLongestDayLength()];
		for (int i = 0; i < timetable.length; i++) {
			timetable[i] = 0;
		}
		photoperiod = Config.getDayLength(World.getDate());
		territories = new HashSet<Long>();
		disturbanceTimes = new TreeSet<Integer>();
		if (getStateClass().getName().contains("WoodpigeonWood")) {
			meanNumberOfTerritories = (Double) Config.getParameter("WoodpigeonLandscapeUnit_territoriesNumberA")
					* Math.pow(getSize(), (Double) Config.getParameter("WoodpigeonLandscapeUnit_territoryNumberB"));
		} else {
			meanNumberOfTerritories = 0;
		}
		resetDisturbanceTimes();
	}

	private void resetTimetable() {
		for (int i = 0; i < photoperiod; i++) {
			timetable[i] = 0;
		}
		photoperiod = Config.getDayLength(World.getDate());
	}

	private void resetDisturbanceTimes() {
		disturbanceTimes.clear();
		int limit = Math.min((Integer) Config.getParameter("WoodpigeonLandscapeUnit_numberOfDisturbances"), photoperiod);
		while (disturbanceTimes.size() < limit-1) {
			disturbanceTimes.add(Config.getGenerator().nextInt(photoperiod));
		}
	}

	/**
	 *
	 * @param time
	 * @return
	 */
	public int getFlockSize(int time) {
//		return timetable[time].size();
		return timetable[time];
	}

	/**
	 *
	 * @param startTime
	 * @return
	 */
	public int getPossibleFeedingTime(int startTime) {
		if (disturbanceTimes.isEmpty()) {
			return photoperiod;
		} else {
			Integer feedingEnd = disturbanceTimes.ceiling(startTime);
			if (feedingEnd != null) {
				return feedingEnd - startTime;
			} else {
				return photoperiod - startTime + disturbanceTimes.first();
			}
		}
	}

	@Override
	public Resource giveResource(HashMap<String, Object> parameters) {
		for (int i : ((Set<Integer>) parameters.get("realForagingInterval"))) {
//			(timetable[i]).add((Long) parameters.get("birdId"));
			timetable[i]++;
		}
		for (Resource r : super.giveResources(null)) {
			if (r.getId() == (Long) parameters.get("resourceId")) {
				return r.splitResource((Double) parameters.get("resourceAmount"));
			}
		}
		return null;
	}

	/**
	 * Returns the Landscape Unit chosen as place for territory or null if the
	 * territory could not be obtained there.
	 *
	 * Adds the bird id to the collection of territory owners.
	 *
	 * @param birdId asking for the territory
	 * @return this landscape unit if the territory is available, otherwise null
	 * @since 2012-06-28
	 */
	public LandscapeUnit getTerritory(long birdId) {
		if (getStateClass().getSimpleName().equalsIgnoreCase("WoodpigeonWood")) {
			double probability = calculateTerritoryGettingProbability(territories.size() / meanNumberOfTerritories);
			if (Config.getGenerator().nextDouble() <= probability) {
				territories.add(birdId);
				return this;
			}
		}
		return null;
	}

	/**
	 * Checks the availability of partners.
	 *
	 * @return id of potential partner (male with a territory hold here without
	 * a nest or partner).
	 * @since 2012-06-28
	 */
	@Override
	public Long providePartnerId() {
		for (long i : territories) {
			Animal a = (Animal) World.Instance().getAgent(i);
			if (a != null) {
				if (a.getPartner() == null && a.getNest() == null) {
					return i;
				}
			}
		}
		return null;
	}

	@Override
	public Signature getSignature() {
		Signature newSignature = super.getSignature();
		String flockSizes = "'{";
		for (int i = 0; i < photoperiod; i++) {
			flockSizes += String.valueOf(timetable[i]) + ",";
		}
		if (flockSizes.equalsIgnoreCase("'{")) {
			flockSizes += "}'";
		} else {
			flockSizes = flockSizes.substring(0, flockSizes.length() - 1)
					+ "}'";
		}
		newSignature.addPrimitiveField("flockSizes", "integer[]", flockSizes);
		newSignature.addPrimitiveField("numberofTerritories", "integer", Integer.toString(territories.size()));
		return newSignature;
	}
	
	/**
	 * @verification TEST 1 Warning if the the state name passed in parameters
	 * is not implemented.
	 */
	@Override
	public AgentState createAgentState(HashMap<String, Object> parameters) {
		if (((String) parameters.get("stateName")).equalsIgnoreCase("Winter_Cereals")) {
			return new WoodpigeonWinterCereals(parameters);
		} else if (((String) parameters.get("stateName")).equalsIgnoreCase("Spring_Cereals")) {
			return new WoodpigeonSpringCereals(parameters);
		} else if (((String) parameters.get("stateName")).equalsIgnoreCase("Spring_Rape")) {
			return new WoodpigeonSpringOilseedRape(parameters);
		} else if (((String) parameters.get("stateName")).equalsIgnoreCase("Winter_Rape")) {
			return new WoodpigeonWinterOilseedRape(parameters);
		} else if (((String) parameters.get("stateName")).equalsIgnoreCase("Pasture")) {
			return new WoodpigeonPasture(parameters);
		} else if (((String) parameters.get("stateName")).equalsIgnoreCase("Clover")) {
			return new WoodpigeonClover(parameters);
		} else if (((String) parameters.get("stateName")).equalsIgnoreCase("Wood")) {
			return new WoodpigeonWood(parameters);
		} else if (((String) parameters.get("stateName")).equalsIgnoreCase("Fallow")) {
			return new WoodpigeonFallow(parameters);
		} else if (((String) parameters.get("stateName")).equalsIgnoreCase("Peas_And_Beans")) {
			return new WoodpigeonPeasAndBeans(parameters);
		} else if (((String) parameters.get("stateName")).equalsIgnoreCase("Brassicaceae")) {
			return new WoodpigeonBrassicaceae(parameters);
		} else if (((String) parameters.get("stateName")).equalsIgnoreCase("Other")) {
			return new WoodpigeonOther(parameters);
		} else {
// *****VERIFICATION MODE START*****//
			if (World.verificationMode) {
				// ***TEST 1
				try {
					throw new SkyPigException(
							"Improper state name: " + ((String) parameters.get("stateName")));
				} catch (SkyPigException ex) {
					World.log(Level.WARNING, Config.getLogInfo(), ex);
				}
			}
		}
// *****VERIFICATION MODE END*****//
		return null;
	}
}
